import sys
import inspect
import logging

from pulsar import HttpException
from pulsar.apps.wsgi import WsgiRequest
from pulsar.utils.tools import checkarity

__all__ = ['RpcHandler', 'rpc_method', 'InvalidRequest', 'InvalidParams',
           'NoSuchFunction', 'InternalError']

LOGGER = logging.getLogger('pulsar.rpc')
_exceptions = {}

def rpc_exception(cls):
    global _exceptions
    code = cls.fault_code
    _exceptions[code] = cls
    return cls

@rpc_exception
class InvalidRequest(HttpException):
    status = 400
    fault_code = -32600
    msg = 'Invalid RPC request'
    

@rpc_exception    
class ParseExcetion(InvalidRequest):
    fault_code = -32700
    msg = 'Parse error'
    

@rpc_exception
class NoSuchFunction(InvalidRequest):
    fault_code = -32601
    msg = 'The method does not exist'


@rpc_exception
class InvalidParams(InvalidRequest):
    fault_code = -32602
    msg = 'Invalid method parameters'
    
    
@rpc_exception
class InternalError(InvalidRequest):
    fault_code = -32603
    msg = 'Internal error'


def exception(code, msg):
    global _exceptions
    cls = _exceptions.get(code, Exception)
    raise cls(msg)
    
def wrap_object_call(fname, namefunc):
    def _(self,*args,**kwargs):
        f = getattr(self, fname)
        return f(*args,**kwargs)
    _.__name__ = namefunc
    return _

def rpcerror(func, args, kwargs, discount=0):
    msg = checkarity(func, args, kwargs, discount=discount)
    if msg:
        raise InvalidParams('Invalid Parameters. %s' % msg)
    else:
        raise
    
def rpc_method(func, doc=None, format='json', request_handler=None):
    '''A decorator which exposes a function ``func`` as an rpc function.

:parameter func: The function to expose.
:parameter doc: Optional doc string. If not provided the doc string of
    ``func`` will be used.
:parameter format: Optional output format.
:parameter request_handler: function which takes ``request``, ``format`` and
    ``kwargs`` and return a new ``kwargs`` to be passed to ``func``.
    It can be used to add additional parameters based on request and format.
    '''
    def _(self, *args, **kwargs):
        request = args[0]
        if request_handler:
            kwargs = request_handler(request, format, kwargs)
        request.format = kwargs.pop('format', format)
        try:
            return func(*args, **kwargs)
        except TypeError:
            rpcerror(func, args, kwargs)
        
    _.__doc__ = doc or func.__doc__
    _.__name__ = func.__name__
    _.FromApi = True
    return _


class MetaRpcHandler(type):
    '''A metaclass for rpc handlers.
Add a limited ammount of magic to RPC handlers.'''
    def __new__(cls, name, bases, attrs):
        make = super(MetaRpcHandler, cls).__new__
        if attrs.pop('virtual', None):
            return make(cls, name, bases, attrs)
        funcprefix = attrs.get('serve_as')
        if not funcprefix:
            for base in bases[::-1]:
                if isinstance(base, MetaRpcHandler):
                    funcprefix = base.serve_as
                    if funcprefix:
                        break
        if funcprefix:
            rpc = set()
            fprefix = '%s_' % funcprefix
            n = len(fprefix)
            for key, method in list(attrs.items()):
                if hasattr(method, '__call__') and key.startswith(fprefix):
                    method_name = key[n:]
                    rpc.add(method_name)
            for base in bases[::-1]:
                if hasattr(base, 'rpc_methods'):
                    rpc.update(base.rpc_methods)
            attrs['rpc_methods'] = frozenset(rpc)
        return make(cls, name, bases, attrs)
    
    
class RpcHandler(MetaRpcHandler('_RpcHandler', (object,), {'virtual': True})):
    serve_as     = 'rpc'
    '''Prefix for class methods providing remote services. Default: ``rpc``.'''
    separator    = '.'
    '''HTTP method allowed by this handler.'''
    virtual = True

    def __init__(self, subhandlers=None, title=None, documentation=None):
        self._parent = None
        self.subHandlers = {}
        self.title = title or self.__class__.__name__
        self.documentation = documentation or self.__doc__
        if subhandlers:
            for prefix,handler in subhandlers.items():
                if inspect.isclass(handler):
                    handler = handler()
                self.putSubHandler(prefix, handler)

    @property
    def parent(self):
        '''The parent :class:`RpcHandler` or ``None`` if this
is the root handler.'''
        return self._parent

    @property
    def root(self):
        '''The root :class:`RpcHandler` or ``self`` if this
is the root handler.'''
        return self._parent.root if self._parent is not None else self

    def isroot(self):
        '''``True`` if this is the root handler.'''
        return self._parent == None

    def putSubHandler(self, prefix, handler):
        '''Add a sub :class:`RpcHandler` with prefix ``prefix``.

:keyword prefix: a string defining the prefix of the subhandler
:keyword handler: the sub-handler.
        '''
        self.subHandlers[prefix] = handler
        handler._parent = self
        return self

    def getSubHandler(self, prefix):
        '''Get a sub :class:`RpcHandler` at ``prefix``.'''
        return self.subHandlers.get(prefix)

    def wrap_function_decorator(self, request, *args, **kwargs):
        return request.func(rpc.handler, request, *args,**kwargs)

    def get_handler(self, method):
        if not method:
            raise NoSuchFunction('RPC method not supplied')
        bits = method.split(self.separator, 1)
        handler = self
        method_name = bits[-1]
        for bit in bits[:-1]:
            subhandler = handler.getSubHandler(bit)
            if subhandler is None:
                method_name = method
                break
            else:
                handler = subhandler
        if method_name in handler.rpc_methods:
            return getattr(handler, '%s_%s' % (self.serve_as, method_name))
        else:
            raise NoSuchFunction('RPC method "%s" not available.' % method)

    def listFunctions(self, prefix=''):
        for name in sorted(self.rpc_methods):
            method = getattr(self, '%s_%s' % (self.serve_as, name))
            docs = method.__doc__ or 'No docs'
            doc = {'doc': ' '.join(docs.split('\n')), 'section': prefix}
            yield '%s%s' % (prefix, name), doc
        for name, handler in self.subHandlers.items():
            pfx = '%s%s%s' % (prefix, name, self.separator)
            for f, doc in handler.listFunctions(pfx):
                yield f, doc

    def _docs(self):
        for name, data in self.listFunctions():
            link = '.. _functions-{0}:'.format(name)
            title = name
            under = (2+len(title))*'-'
            yield '\n'.join((link,'',title,under,'',data['doc'],'\n'))

    def docs(self):
        return '\n'.join(self._docs())



from django.conf.urls import patterns, url, include
from django.contrib import admin
from django.contrib.staticfiles.urls import staticfiles_urlpatterns


admin.autodiscover()


urlpatterns = patterns('',
    url(r'^$', 'chat.views.home'),
    url(r'^admin/', include(admin.site.urls))
)

urlpatterns += staticfiles_urlpatterns()


import math
import time
from datetime import timedelta
from random import random
from functools import reduce

from pulsar.apps.tasks import PeriodicJob,  anchorDate


class TestPeriodicJob(PeriodicJob):
    abstract = True
    run_every = timedelta(hours=1)


class TestPeriodic(TestPeriodicJob):
    run_every = timedelta(seconds=5)
    def __call__(self, consumer):
        return 'OK'
    

class TestPeriodicError(TestPeriodicJob):
    run_every = timedelta(seconds=60)
    def __call__(self, consumer):
        raise Exception('kaputt')
    

class AnchoredEveryHour(TestPeriodicJob):
    anchor = anchorDate(minute=25)
    def __call__(self, consumer):
        raise Exception('kaputt')
    
    
class StandardDeviation(TestPeriodicJob):
    
    def can_overlap(self, inputs=None, **kwargs):
        return inputs is not None
    
    def __call__(self, consumer, inputs=None, sample=10, size=100):
        if inputs is None:
            for n in range(sample):
                inputs = [random() for i in range(size)]
                self.run_job(consumer, self.name, inputs=inputs)
            return 'produced %s new tasks' % sample
        else:
            time.sleep(0.1)
            v2 = reduce(lambda x,y: x+y, map(lambda x: x*x, inputs))/len(inputs)
            return math.sqrt(v2)
            
        



'''

.. autoclass:: HttpServerResponse
   :members:
   :member-order: bysource
   
'''
import sys
import time
import os
import socket
from functools import partial
from wsgiref.handlers import format_date_time
from io import BytesIO

import pulsar
from pulsar import HttpException, ProtocolError, maybe_async, is_async
from pulsar.utils.pep import is_string, to_bytes, native_str
from pulsar.utils.httpurl import Headers, unquote, has_empty_content,\
                                 host_and_port_default, Headers, http_parser,\
                                 REDIRECT_CODES
from pulsar.utils import events

from .utils import handle_wsgi_error, LOGGER, HOP_HEADERS
from .wrappers import WsgiResponse


__all__ = ['HttpServerResponse', 'MAX_CHUNK_SIZE']

MAX_CHUNK_SIZE = 65536


def chunk_encoding(chunk):
    '''Write a chunk::

    chunk-size(hex) CRLF
    chunk-data CRLF
    
If the size is 0, this is the last chunk, and an extra CRLF is appended.
'''
    head = ("%X\r\n" % len(chunk)).encode('utf-8')
    return head + chunk + b'\r\n'

def keep_alive(headers, version):
        """ return True if the connection should be kept alive"""
        conn = set((v.lower() for v in headers.get_all('connection', ())))
        if "close" in conn:
            return False
        elif 'upgrade' in conn:
            headers['connection'] = 'Upgrade'
            return True
        elif "keep-alive" in conn:
            return True
        elif version == (1, 1):
            headers['connection'] = 'keep-alive'
            return True
        else:
            return False
        
def keep_alive_with_status(status, headers):
    code = int(status.split()[0])
    if code >= 400:
        return False
    return True
    

class HttpServerResponse(pulsar.ProtocolConsumer):
    '''Server side HTTP :class:`pulsar.ProtocolConsumer`.'''
    _status = None
    _headers_sent = None
    _request_headers = None
    max_empty_consecutive = 20
    SERVER_SOFTWARE = pulsar.SERVER_SOFTWARE
    
    def __init__(self, wsgi_callable, cfg, connection):
        super(HttpServerResponse, self).__init__(connection)
        self.wsgi_callable = wsgi_callable
        self.cfg = cfg
        self.parser = http_parser(kind=0)
        self.headers = Headers()
        self.keep_alive = False
        
    def data_received(self, data):
        '''Implements :class:`pulsar.Protocol.data_received`. Once we have a
full HTTP message, build the wsgi ``environ`` and write the response
using the :meth:`pulsar.Transport.writelines` method.'''
        p = self.parser
        request_headers = self._request_headers
        if p.execute(bytes(data), len(data)) == len(data):
            done = p.is_message_complete()
            if request_headers is None and p.is_headers_complete():
                self._request_headers = Headers(p.get_headers(), kind='client')
                if not done:
                    self.expect_continue()
            if done: # message is done
                self.generate(self.wsgi_environ())
        else:
            # This is a parsing error, the client must have sent
            # bogus data
            raise ProtocolError
    
    def expect_continue(self):
        '''Handle the expect=100-continue header if available, according to
the following algorithm:

* Send the 100 Continue response before waiting for the body.
* Omit the 100 (Continue) response if it has already received some or all of
  the request body for the corresponding request.
    '''
        if self._request_headers.has('expect', '100-continue'):
            self.transport.write(b'HTTP/1.1 100 Continue\r\n\r\n')
    
    @property
    def status(self):
        return self._status

    @property
    def upgrade(self):
        return self.headers.get('Upgrade')

    @property
    def chunked(self):
        return self.headers.get('Transfer-Encoding') == 'chunked'

    @property
    def content_length(self):
        c = self.headers.get('Content-Length')
        if c:
            return int(c)

    @property
    def version(self):
        return self.parser.get_version()

    def start_response(self, status, response_headers, exc_info=None):
        '''WSGI compliant ``start_response`` callable, see pep3333_.
The application may call start_response more than once, if and only
if the ``exc_info`` argument is provided.
More precisely, it is a fatal error to call start_response without the exc_info
argument if start_response has already been called within the current
invocation of the application.

:parameter status: an HTTP "status" string like "200 OK" or "404 Not Found".
:parameter response_headers: a list of ``(header_name, header_value)`` tuples.
    It must be a Python list. Each header_name must be a valid HTTP header
    field-name (as defined by RFC 2616_, Section 4.2), without a trailing
    colon or other punctuation.
:parameter exc_info: optional python ``sys.exc_info()`` tuple. This argument
    should be supplied by the application only if start_response is being
    called by an error handler.

:rtype: The :meth:`HttpResponse.write` callable.

The ``HOP_HEADERS`` are not considered but no error is raised.

.. _pep3333: http://www.python.org/dev/peps/pep-3333/
.. _2616: http://www.faqs.org/rfcs/rfc2616.html
'''
        if exc_info:
            try:
                if self._headers_sent:
                    # if exc_info is provided, and the HTTP headers have
                    # already been sent, start_response must raise an error,
                    # and should re-raise using the exc_info tuple
                    raise (exc_info[0], exc_info[1], exc_info[2])
            finally:
                # Avoid circular reference
                exc_info = None
        elif self._status:
            # Headers already set. Raise error
            raise HttpException("Response headers already set!")
        self._status = status
        if type(response_headers) is not list:
            raise TypeError("Headers must be a list of name/value tuples")
        for header, value in response_headers:
            if header.lower() in HOP_HEADERS:
                # These features are the exclusive province of this class,
                # this should be considered a fatal error for an application
                # to attempt sending them, but we don't raise an error,
                # just log a warning
                LOGGER.warning('Application handler passing hop header "%s"',
                               header)
                continue
            self.headers.add_header(header, value)
        return self.write

    def write(self, data, force=False):
        '''The write function which is returned by the :meth:`start_response`
method as required by the WSGI specification.

:param data: bytes to write
:param force: Optional flag used internally.
'''
        if not self._headers_sent:
            tosend = self.get_headers()
            events.fire('http-headers', self, headers=tosend)
            self._headers_sent = tosend.flat(self.version, self.status)
            self.transport.write(self._headers_sent)
        if data:
            if self.chunked:
                while len(data) >= MAX_CHUNK_SIZE:
                    chunk, data = data[:MAX_CHUNK_SIZE], data[MAX_CHUNK_SIZE:]
                    self.transport.write(chunk_encoding(chunk))
                if data:
                    self.transport.write(chunk_encoding(data))
            else:
                self.transport.write(data)
        elif force and self.chunked:
            self.transport.write(chunk_encoding(data))

    def generate(self, environ, failure=None):
        try:
            if failure:
                # A failure has occurred, try to handle it with the default
                # wsgi error handler
                wsgi_iter = self.handle_wsgi_error(environ, failure)
            else:
                wsgi_iter = self.wsgi_callable(environ, self.start_response)
        except Exception:
            result = sys.exc_info()
        else:
            result = self.async_wsgi(wsgi_iter)
        result = maybe_async(result, get_result=False)
        err_handler = self.catastrofic_failure if failure else self.generate
        result.add_errback(partial(err_handler, environ))
        
    def async_wsgi(self, wsgi_iter):
        '''Asynchronous WSGI server handler. Fully conforms with `WSGI 1.0.1`_
when the ``wsgi_iter`` yields bytes only.'''
        if is_async(wsgi_iter):
            wsgi_iter = yield wsgi_iter 
        try:
            for b in wsgi_iter:
                chunk = yield b # handle asynchronous components
                self.write(chunk)
            # make sure we write headers
            self.write(b'', True)
        finally:
            if hasattr(wsgi_iter, 'close'):
                try:
                    wsgi_iter.close()
                except Exception:
                    LOGGER.exception('Error while closing wsgi iterator')
        self.finish_wsgi()
        
    def handle_wsgi_error(self, environ, failure):
        exc_info = failure.trace
        response = handle_wsgi_error(environ, exc_info)
        self.start_response(response.status, response.get_headers(), exc_info)
        return response
        
    def catastrofic_failure(self, environ, failure):
        response = WsgiResponse(500)
        self.start_response(response.status, response.get_headers(), exc_info)
        return response
        
    def finish_wsgi(self):
        if not self.keep_alive:
            self.connection.close()
        self.finished()
            
    def is_chunked(self):
        '''Only use chunked responses when the client is
speaking HTTP/1.1 or newer and there was no Content-Length header set.'''
        if self.version <= (1, 0):
            return False
        elif has_empty_content(int(self.status[:3])):
            # Do not use chunked responses when the response
            # is guaranteed to not have a response body.
            return False
        elif self.headers.get('Transfer-Encoding') == 'chunked':
            return True
        else:
            return self.content_length is None

    def get_headers(self):
        '''Get the headers to send only if *force* is ``True`` or this
is an HTTP upgrade (websockets)'''
        if not self._status:
            # we are sending headers but the start_response was not called
            raise HttpException('Headers not set.')
        headers = self.headers
        # Set chunked header if needed
        if self.is_chunked():
            headers['Transfer-Encoding'] = 'chunked'
            headers.pop('content-length', None)
        else:
            headers.pop('Transfer-Encoding', None)
        if self.keep_alive:
            self.keep_alive = keep_alive_with_status(self._status, headers)
        if not self.keep_alive:
            headers['connection'] = 'close'
        return headers

    def wsgi_environ(self):
        #return a the WSGI environ dictionary
        p = self.parser
        input = BytesIO(p.recv_body())
        protocol = "HTTP/%s" % ".".join(('%s' % v for v in p.get_version()))
        environ = {
            "wsgi.input": input,
            "wsgi.errors": sys.stderr,
            "wsgi.version": (1, 0),
            "wsgi.run_once": False,
            'wsgi.multithread': False,
            'wsgi.multiprocess': False,
            "SERVER_SOFTWARE": pulsar.SERVER_SOFTWARE,
            "REQUEST_METHOD": native_str(p.get_method()),
            "QUERY_STRING": p.get_query_string(),
            "RAW_URI": p.get_url(),
            "SERVER_PROTOCOL": protocol,
            'CONTENT_TYPE': '',
            'pulsar.connection': self.connection,
            'pulsar.cfg': self.cfg
        }
        url_scheme = "http"
        forward = self.address
        server = '%s:%s' % self.transport.address
        script_name = os.environ.get("SCRIPT_NAME", "")
        for header, value in self._request_headers:
            header = header.lower()
            if header in HOP_HEADERS:
                self.headers[header] = value
            if header == 'x-forwarded-for':
                forward = value
            elif header == "x-forwarded-protocol" and value == "ssl":
                url_scheme = "https"
            elif header == "x-forwarded-ssl" and value == "on":
                url_scheme = "https"
            elif header == "host":
                server = value
            elif header == "script_name":
                script_name = value
            elif header == "content-type":
                environ['CONTENT_TYPE'] = value
                continue
            elif header == "content-length":
                environ['CONTENT_LENGTH'] = value
                continue
            key = 'HTTP_' + header.upper().replace('-', '_')
            environ[key] = value
        environ['wsgi.url_scheme'] = url_scheme
        if is_string(forward):
            # we only took the last one
            # http://en.wikipedia.org/wiki/X-Forwarded-For
            if forward.find(",") >= 0:
                forward = forward.rsplit(",", 1)[1].strip()
            remote = forward.split(":")
            if len(remote) < 2:
                remote.append('80')
        else:
            remote = forward
        environ['REMOTE_ADDR'] = remote[0]
        environ['REMOTE_PORT'] = str(remote[1])
        server =  host_and_port_default(url_scheme, server)
        environ['SERVER_NAME'] = socket.getfqdn(server[0])
        environ['SERVER_PORT'] = server[1]
        path_info = p.get_path()
        if path_info is not None:
            if script_name:
                path_info = path_info.split(script_name, 1)[1]
            environ['PATH_INFO'] = unquote(path_info)
        environ['SCRIPT_NAME'] = script_name
        self.keep_alive = keep_alive(self.headers, p.get_version())
        self.headers.update([('Server', self.SERVER_SOFTWARE),
                             ('Date', format_date_time(time.time()))])
        return environ


import os
import sys
import logging
import time

import pythoncom
import win32serviceutil
import win32service
import win32event
import win32api
import servicemanager

import pulsar
from pulsar.utils.importer import import_module

import multiprocessing

from .winprocess import WINEXE


class ServiceManagerLogHandler(logging.Handler):
    
    def emit(self, record):
        try:
            msg = self.format(record)
            if record.levelno >= logging.ERROR:
                servicemanager.LogErrorMsg(msg)
            elif record.levelno >= logging.INFO:
                servicemanager.LogiNFOMsg(msg)
        except Exception:
            pass
            

def ctrlHandler(ctrlType):
   return True


class PulsarService(win32serviceutil.ServiceFramework):
    _svc_name_ = 'PULSAR_%s' % pulsar.__version__
    _svc_display_name_ = "PULSAR %s server" % pulsar.__version__
    _svc_description_ = "Pulsar asynchronous server"
    _command_lines = 'pulsar_command_line'
    
    def __init__(self, args):
        win32serviceutil.ServiceFramework.__init__(self,args)
        self.hWaitStop = win32event.CreateEvent(None,0,0,None)
        self.running = False

    def SvcStop(self):
        #self.log.info('%s - Received stop signal' % self._svc_name_)
        pulsar.arbiter().stop()
        self.running = False
        self.ReportServiceStatus(win32service.SERVICE_STOP_PENDING)
        win32event.SetEvent(self.hWaitStop)

    def SvcDoRun(self):
        mod = import_module(self._command_lines)
        os.remove(os.path.abspath(mod.__file__))
        self.setup(mod.argv)
    
    def setup(self,argv):
        raise NotImplementedError
        
    def main(self):
        arbiter = pulsar.arbiter()
        arbiter.log.info('%s - starting up the service' % self._svc_name_)
        self.running = True
        while self.running:
            time.sleep(1)
        arbiter.log.info('%s - exiting up the service' % self._svc_name_)
    
    @classmethod
    def run(cls, **params):
        argv = sys.argv
        args = sys.argv[:1]
        if len(sys.argv) > 2:
            args += sys.argv[2:]
        f = open(cls._command_lines+'.py','w')
        f.write('argv = {0}'.format(args))
        f.close()
        argv = argv[:2]
        if not argv or not argv[0]:
            main_path = getattr(sys.modules['__main__'], '__file__', None)
            path,script = os.path.split(main_path)
            argv = [script,'start']
        sys.argv = argv
        cls.params = params
        win32api.SetConsoleCtrlHandler(ctrlHandler, True)
        win32serviceutil.HandleCommandLine(cls)


    
if __name__ == '__main__':
    '''To debug type::

    python winservice.py debug
    '''
    PulsarService.run()



"""This is the most important :ref:`pulsar application <application-api>`.
The server is a specialized :ref:`socket server <apps-socket>`
for web applications conforming with the python web server
gateway interface (`WSGI 1.0.1`_).
The server can be used in conjunction with several web frameworks
as well as :ref:`pulsar wsgi application handlers <wsgi-handlers>`,
the :ref:`pulsar RPC middleware <apps-rpc>` and
the :ref:`websocket middleware <apps-ws>`.

An example of a web server written with :mod:`pulsar.apps.wsgi` which responds
with "Hello World!" for every request::

    from pulsar.apps import wsgi

    def hello(environ, start_response):
        data = b"Hello World!"
        response_headers = [('Content-type','text/plain'),
                            ('Content-Length', str(len(data)))]
        start_response("200 OK", response_headers)
        return [data]

    if __name__ == '__main__':
        wsgi.WSGIServer(callable=hello).start()


For more information regarding WSGI check the pep3333_ specification.
To run the application::

    python script.py
    
For available run options::

    python script.py --help


WSGI Server
===================

.. autoclass:: WSGIServer
   :members:
   :member-order: bysource
   
   
.. _`WSGI 1.0.1`: http://www.python.org/dev/peps/pep-3333/
"""
from functools import partial

import pulsar
from pulsar.apps.socket import SocketServer

from .content import *
from .utils import *
from .middleware import *
from .wrappers import *
from .server import *
from .route import *
from .handlers import *
from .plugins import *


class WsgiFactory(object):
    
    def __call__(self):
        raise NotImplementedError


class WSGIServer(SocketServer):
    name = 'wsgi'
    cfg = pulsar.Config(apps=['socket', 'wsgi'])

    def protocol_consumer(self):
        '''Build the :class:`pulsar.ProtocolConsumer` factory for this
WSGI server. It uses the :class:`pulsar.apps.wsgi.server.HttpServerResponse`
protocol consumer and the wsgi callable provided as parameter during
initialisation.'''
        callable = self.callable
        if isinstance(callable, WsgiFactory):
            callable = callable()
        return partial(HttpServerResponse, callable, self.cfg)



'''Utility classes for managing Header Authentication, used both by
servers and clients.

header parser
~~~~~~~~~~~~~~~~~~~~~~

.. autofunction:: parse_authorization_header


Auth
~~~~~~~~~~~~~~~~~

.. autoclass:: Auth
   :members:
   :member-order: bysource
   
   
HTTPBasicAuth
~~~~~~~~~~~~~~~~~

.. autoclass:: HTTPBasicAuth
   :members:
   :member-order: bysource
   
HTTPDigestAuth
~~~~~~~~~~~~~~~~~

.. autoclass:: HTTPDigestAuth
   :members:
   :member-order: bysource
   
'''
from base64 import b64encode, b64decode

import pulsar
from pulsar.utils.httpurl import parse_dict_header
from pulsar.utils.pep import native_str

__all__ = ['Auth', 
           'HTTPBasicAuth',
           'HTTPDigestAuth',
           'basic_auth_str',
           'parse_authorization_header']
        

class Auth(object):
    """Base class for managing HTTP header authentication"""
    type = None
    def __call__(self, r):
        raise NotImplementedError('Auth hooks must be callable.')

    def authenticated(self, *args, **kwargs):
        return False

    def __str__(self):
        return self.__repr__()


class HTTPBasicAuth(Auth):
    """Attaches HTTP Basic Authentication to the given Request object."""
    def __init__(self, username, password):
        self.username = username
        self.password = password
        
    @property
    def type(self):
        return 'basic'

    def __call__(self, request):
        request.headers['Authorization'] = basic_auth_str(self.username,
                                                          self.password)

    def authenticated(self, environ, username, password):
        return username==self.username and password==self.password

    def __repr__(self):
        return 'Basic: %s' % self.username


class HTTPDigestAuth(Auth):
    """Attaches HTTP Digest Authentication to the given Request object."""
    def __init__(self, username, password=None, options=None):
        self.username = username
        self.password = password
        self.last_nonce = None
        self.options = options or {}
        self.algorithm = self.options.pop('algorithm', 'MD5')
    
    @property
    def type(self):
        return 'digest'
    
    def __call__(self, request):
        # If we have a saved nonce, skip the 401
        if self.last_nonce:
            request.headers['Authorization'] =\
                self.encode(request.method, request.full_url)
        request.register_hook('post_request', self.handle_401)
        return request

    def __repr__(self):
        return 'Digest: %s' % self.username

    def authenticated(self, environ, username, password):
        '''Called by the server to check if client is authenticated.'''
        if username != self.username:
            return False
        o = self.options
        qop = o.get('qop')
        method = environ['REQUEST_METHOD']
        uri = environ.get('PATH_INFO','')
        ha1 = self.ha1(o['realm'], password)
        ha2 = self.ha2(qop, method, uri)
        if qop is None:
            response = hexmd5(":".join((ha1, self.nonce, ha2)))
        elif qop == 'auth' or qop == 'auth-int':
            response = hexmd5(":".join((ha1, o['nonce'], o['nc'],
                                        o['cnonce'], qop, ha2)))
        else:
            raise ValueError("qop value are wrong")
        return o['response'] == response

    def encode(self, method, uri):
        '''Called by the client to encode Authentication header.'''
        if not self.username or not self.password:
            return
        o = self.options
        qop = o.get('qop')
        realm = o.get('realm')
        nonce = o['nonce']
        entdig = None
        p_parsed = urlparse(uri)
        path = p_parsed.path
        if p_parsed.query:
            path += '?' + p_parsed.query
        KD = lambda s, d: self.hex("%s:%s" % (s, d))
        ha1 = self.ha1(realm, self.password)
        ha2 = self.ha2(qop, method, path)
        if qop == 'auth':
            if nonce == self.last_nonce:
                self.nonce_count += 1
            else:
                self.nonce_count = 1
            ncvalue = '%08x' % self.nonce_count
            s = str(self.nonce_count).encode('utf-8')
            s += nonce.encode('utf-8')
            s += time.ctime().encode('utf-8')
            s += os.urandom(8)
            cnonce = sha1(s).hexdigest()[:16]
            noncebit = "%s:%s:%s:%s:%s" % (nonce, ncvalue, cnonce, qop, ha2)
            respdig = KD(ha1, noncebit)
        elif qop is None:
            respdig = KD(ha1, "%s:%s" % (nonce, ha2))
        else:
            # XXX handle auth-int.
            return
        base = 'username="%s", realm="%s", nonce="%s", uri="%s", ' \
               'response="%s"' % (self.username, realm, nonce, path, respdig)
        opaque = o.get('opaque')
        if opaque:
            base += ', opaque="%s"' % opaque
        if entdig:
            base += ', digest="%s"' % entdig
            base += ', algorithm="%s"' % self.algorithm
        if qop:
            base += ', qop=%s, nc=%s, cnonce="%s"' % (qop, ncvalue, cnonce)
        return 'Digest %s' % (base)
        
    def handle_401(self, response):
        """Takes the given response and tries digest-auth, if needed."""
        request = response.request
        num_401_calls = request.hooks['response'].count(self.handle_401)
        s_auth = response.headers.get('www-authenticate', '')
        if 'digest' in s_auth.lower() and num_401_calls < 2:
            self.options = parse_dict_header(s_auth.replace('Digest ', ''))
            request.headers['Authorization'] = self.encode(request.method,
                                                           request.full_url)
            response.again = True
        return response
    
    def hex(self, x):
        if self.algorithm == 'MD5':
            return hexmd5(x)
        elif self.algorithm == 'SHA1':
            return hexsha1(x)
        else:
            raise valueError('Unknown algorithm %s' % self.algorithm)
        
    def ha1(self, realm, password):
        return self.hex('%s:%s:%s' % (self.username, realm, password))
    
    def ha2(self, qop, method, uri, body=None):
        if qop == "auth" or qop is None:
            return self.hex("%s:%s" % (method, uri))
        elif qop == "auth-int":
            return self.hex("%s:%s:%s" % (method, uri, self.hex(body)))
        raise ValueError()


def basic_auth_str(username, password):
    """Returns a Basic Auth string."""
    b64 = b64encode(('%s:%s' % (username, password)).encode('latin1'))
    return 'Basic %s' % native_str(b64.strip(), 'latin1')

digest_parameters=frozenset(('username', 'realm', 'nonce', 'uri', 'nc',
                             'cnonce',  'response'))
def parse_authorization_header(value, charset='utf-8'):
    """Parse an HTTP basic/digest authorization header transmitted by the web
browser.  The return value is either `None` if the header was invalid or
not given, otherwise an :class:`Auth` object.

:param value: the authorization header to parse.
:return: a :class:`Auth` or `None`."""
    if not value:
        return
    try:
        auth_type, auth_info = value.split(None, 1)
        auth_type = auth_type.lower()
    except ValueError:
        return
    if auth_type == 'basic':
        try:
            up = b64decode(auth_info.encode('latin-1')).decode(charset)
            username, password = up.split(':', 1)
        except Exception as e:
            return
        return HTTPBasicAuth(username, password)
    elif auth_type == 'digest':
        auth_map = parse_dict_header(auth_info)
        if not digest_parameters.difference(auth_map):
            return HTTPDigestAuth(auth_map.pop('username'),
                                  options=auth_map)



'''Tests the taskqueue redis backend.'''
try:
    import stdnet
except ImportError:
    stdnet = None
    
from pulsar.apps.test import unittest
 
from . import test_local

raise ImportError

unittest.skipUnless(stdnet, 'Requires python-stdnet')
class TestRedisTaskQueueOnThread(test_local.TestTaskQueueOnThread):
    #schedule_periodic = False
    
    @classmethod
    def task_backend(cls):
        return cls.cfg.redis_server


#unittest.skipUnless(stdnet, 'Requires python-stdnet')
#class TestRedisTaskQueueOnProcess(test_app.TestTaskQueueOnProcess):
#    
#    @classmethod
#    def task_backend(cls):
#        return cls.cfg.redis_server



'''TCP protocol clients and servers'''
import socket

from pulsar.utils.sockets import *
from pulsar.utils.pep import range
from pulsar.utils.structures import merge_prefix

from .servers import Server
from .transport import SocketTransport, LOGGER

__all__ = ['TCP']

TRY_WRITE_AGAIN = (EWOULDBLOCK, ENOBUFS, EINPROGRESS)
TRY_READ_AGAIN = (EWOULDBLOCK, EAGAIN)
NUMBER_ACCEPTS = 30 if platform.type == "posix" else 1


class TCP(SocketTransport):
    '''Transport for the TCP protocol.'''
    TYPE = socket.SOCK_STREAM
    
    def _protocol_connect(self, address):
        try:
            self.sock.connect(address)
        except socket.error as e:
            if e.args[0] in TRY_WRITE_AGAIN:
                return False
            else:
                raise
        else:
            return True #    A synchronous connection
    
    def _protocol_read(self):
        try:
            return self._sock.recv(self._read_chunk_size)
        except socket.error as e:
            if e.args[0] == EWOULDBLOCK:
                return
            else:
                raise
            
    def _protocol_write(self):
        buffer = self._write_buffer
        tot_bytes = 0
        if not buffer:
            LOGGER.warning('handling write on a 0 length buffer')
        while buffer:
            try:
                sent = self.sock.send(buffer[0])
                if sent == 0:
                    # With OpenSSL, after send returns EWOULDBLOCK,
                    # the very same string object must be used on the
                    # next call to send.  Therefore we suppress
                    # merging the write buffer after an EWOULDBLOCK.
                    break
                merge_prefix(buffer, sent)
                buffer.popleft()
                tot_bytes += sent
            except socket.error as e:
                if e.args[0] in TRY_WRITE_AGAIN:
                    break
                else:
                    raise
        return tot_bytes
        
    def _protocol_accept(self):
        try:
            for i in range(NUMBER_ACCEPTS):
                if self.closed:
                    return
                try:
                    sock, address = self.sock.accept()
                except socket.error as e:
                    if e.args[0] in TRY_READ_AGAIN:
                        break
                    elif e.args[0] == EPERM:
                        # Netfilter on Linux may have rejected the
                        # connection, but we get told to try to accept() anyway.
                        continue
                    elif e.args[0] in TCP_ACCEPT_ERRORS:
                        LOGGER.info('Could not accept new connection')
                        break
                    raise
                self._protocol.data_received(sock, address)
        except Exception:
            LOGGER.exception('Could not accept new connection')
        


'''The :mod:`pulsar.apps.ws` contains WSGI_ middleware for
handling the WebSocket_ protocol.
Web sockets allow for bidirectional communication between the browser
and server. Pulsar implementation uses the WSGI middleware
:class:`WebSocket` for the handshake_ and a class derived from
:class:`WS` handler for the communication part.

Here is an example Web Socket handler that echos back all received messages
back to the client::

    from pulsar.apps import wsgi, ws
    
    class EchoWS(ws.WS):
    
        def on_open(self, request):
            pass
    
        def on_message(self, request, message):
            return message
    
        def on_close(self, request):
            pass
            
To create a valid :class:`WebSocket` middleware initialise as follow::

    wm = ws.WebSocket('/bla', EchoWS())
    app = wsgi.WsgiHandler(middleware=(..., wm))
    wsgi.WSGIServer(callable=app).start()


.. _WSGI: http://www.python.org/dev/peps/pep-3333/
.. _WebSocket: http://tools.ietf.org/html/rfc6455
.. _handshake: http://tools.ietf.org/html/rfc6455#section-1.3

API
==============

.. _websocket-middleware:

WebSocket
~~~~~~~~~~~~~~~~

.. autoclass:: WebSocket
   :members:
   :member-order: bysource


.. _websocket-handler:

WebSocket handler
~~~~~~~~~~~~~~~~~~~~

.. autoclass:: WS
   :members:
   :member-order: bysource
   

WebSocket protocol
~~~~~~~~~~~~~~~~~~~~

.. autoclass:: WebSocketProtocol
   :members:
   :member-order: bysource
'''
import logging
import socket
import re
import struct
import time
import base64
import hashlib
from functools import partial

import pulsar
from pulsar import async, HttpException, ProtocolError, log_failure
from pulsar.utils.httpurl import to_bytes, native_str
from pulsar.utils.websocket import FrameParser, Frame
from pulsar.apps import wsgi

from . import extensions

WEBSOCKET_GUID = '258EAFA5-E914-47DA-95CA-C5AB0DC85B11'


class GeneralWebSocket(wsgi.Router):
    '''A websocket middleware.
    
.. attribute:: frame_parser

    A factory of websocket frame parsers
'''
    namespace = ''
    frame_parser = FrameParser
    
    def __init__(self, route, handle, frame_parser=None, start_response=False,
                 **kwargs):
        super(GeneralWebSocket, self).__init__(route, **kwargs)
        self.handle = handle
        self.start_response = start_response
        if frame_parser:
            self.frame_parser = frame_parser
        
    def get(self, request):
        headers_parser = self.handle_handshake(request.environ)
        if not headers_parser:
            raise HttpException(status=404)
        headers, parser = headers_parser
        request.response.status_code = 101
        request.response.content = b''
        request.response.headers.update(headers)
        upgrade = request.environ['pulsar.connection'].upgrade
        upgrade(partial(WebSocketServerProtocol, self.handle,
                        request, parser))
        return request.response
    
    def handle_handshake(self, environ):
        '''handle the websocket handshake. Must return a list of HTTP
headers to send back to the client.'''
        raise NotImplementedError
        
    
class WebSocket(GeneralWebSocket):
    """A specialised :ref:`Router <apps-wsgi-router>` middleware for
handling the websocket handshake at a given route.
Once the handshake is succesful, the protocol consumer
is upgraded to :class:`WebSocketProtocol` and messages are handled by
the :attr:`handle` attribute, an instance of :class:`WS`.

See http://tools.ietf.org/html/rfc6455 for the websocket server protocol and
http://www.w3.org/TR/websockets/ for details on the JavaScript interface.
    """        
    def handle_handshake(self, environ):
        connections = environ.get("HTTP_CONNECTION", '').lower()\
                                    .replace(' ','').split(',')
        if environ.get("HTTP_UPGRADE", '').lower() != "websocket" or \
           'upgrade' not in connections:
            return
        if environ['REQUEST_METHOD'].upper() != 'GET':
            raise HttpException(msg='Method is not GET', status=400)
        key = environ.get('HTTP_SEC_WEBSOCKET_KEY')
        if key:
            try:
                ws_key = base64.b64decode(key.encode('latin-1'))
            except Exception:
                ws_key = ''
            if len(ws_key) != 16:
                raise HttpException(msg="WebSocket key's length is invalid",
                                    status=400)
        else:
            raise HttpException(msg='Not a valid HyBi WebSocket request. '
                                    'Missing Sec-Websocket-Key header.',
                                status=400)
        # Collect supported subprotocols
        subprotocols = environ.get('HTTP_SEC_WEBSOCKET_PROTOCOL')
        ws_protocols = []
        if subprotocols:
            for s in subprotocols.split(','):
                ws_protocols.append(s.strip())
        # Collect supported extensions
        ws_extensions = []
        extensions = environ.get('HTTP_SEC_WEBSOCKET_EXTENSIONS')
        if extensions:
            for ext in extensions.split(','):
                ws_extensions.append(ext.strip())
        # Build the frame parser
        version = environ.get('HTTP_SEC_WEBSOCKET_VERSION')
        try:
            parser = self.frame_parser(version=version, protocols=ws_protocols,
                                       extensions=ws_extensions)
        except ProtocolError as e:
            raise HttpException(str(e), status=400)
        headers = [('Sec-WebSocket-Accept', self.challenge_response(key))]
        if parser.protocols:
            headers.append(('Sec-WebSocket-Protocol',
                            ', '.join(parser.protocols)))
        if parser.extensions:
            headers.append(('Sec-WebSocket-Extensions',
                            ','.join(parser.extensions)))
        return headers, parser
        
    def challenge_response(self, key):
        sha1 = hashlib.sha1(to_bytes(key+WEBSOCKET_GUID))
        return native_str(base64.b64encode(sha1.digest()))
    
    def get_parser(self):
        return FrameParser()


class WS(object):
    '''A web socket handler. It maintains
an open socket with a remote web-socket and exchange messages in
an asynchronous fashion. The communication is started by the
:class:`WebSocket` middleware after a successful handshake.
 
Override :meth:`on_message` to handle incoming messages.
You can also override :meth:`on_open` and :meth:`on_close` to handle opened
and closed connections.
These methods accept as first parameter the
:class:`pulsar.apps.wsgi.wrappers.WsgiRequest` created during the handshake.
'''
    def on_open(self, request):
        """Invoked when a new WebSocket is opened."""
        pass

    def on_message(self, request, message):
        '''Handles incoming messages on the WebSocket.
This method must be overloaded. Whatever is returned by this method
is handled by :class:`WebSocketProtocol.write` method.'''
        raise NotImplementedError

    def on_close(self, request):
        """Invoked when the WebSocket is closed."""
        pass
        
    def write(self, request, message):
        '''An utility method for writing a message to the client.
It uses the :class:`WebSocketProtocol` which is accessible from the *request*
parameter. It is a proxy for the :class:`WebSocketProtocol.write` method.'''
        return request.cache['websocket'].write(message)
        
        
class WebSocketProtocol(pulsar.ProtocolConsumer):
    '''Websocket protocol for servers and clients.'''
    request = None
    started = False
    closed = False
    
    def data_received(self, data):
        frame = self.parser.decode(data)
        while frame:
            self.write(self.parser.replay_to(frame))
            if frame.is_close:
                return self.close()
            if not self.started:
                self.started = True
                self.write(self.handler.on_open(self.request))
            if frame.is_data:
                self.write(self.handler.on_message(self.request, frame.body))
            frame = self.parser.decode()
    
    def write(self, value):
        '''Write a new *frame* into the wire, frame can be:

* ``None`` does nothing
* bytes - converted to a byte Frame
* string - converted to a string Frame
* a :class:`pulsar.utils.websocket.Frame`
 '''
        self._async_write(value).add_errback(self.close)
            
    def close(self, error=None):
        if not self.closed:
            log_failure(error)
            self.closed = True
            self.handler.on_close(self.request)
            self.transport.close()
    
    # INTERNAL 
    @async(get_result=False)
    def _async_write(self, value):
        frame = yield value
        if frame is None:
            return
        elif not isinstance(frame, Frame):
            frame = self.parser.encode(frame)
            self.transport.write(frame.msg)
        else:
            self.transport.write(frame.msg)
        if frame.is_close:
            self.close()
    
class WebSocketServerProtocol(WebSocketProtocol):
    '''Created after a successful websocket handshake. Tjis is a
:class:`pulsar.ProtocolConsumer` which manages the communication with the
websocket client.'''
    def __init__(self, handler, request, parser, connection):
        super(WebSocketServerProtocol, self).__init__(connection)
        connection.set_timeout(0)
        self.handler = handler
        self.request = request
        self.parser = parser
        request.cache['websocket'] = self



'''Tests the "helloworld" example.'''
from pulsar import send, SERVER_SOFTWARE, get_application
from pulsar import MultiDeferred
from pulsar.utils.pep import range
from pulsar.apps.http import HttpClient
from pulsar.apps.test import unittest, run_on_arbiter, dont_run_with_thread

from .manage import server
        

class TestHelloWorldThread(unittest.TestCase):
    app = None
    concurrency = 'thread'
    
    @classmethod
    def name(cls):
        return 'helloworld_' + cls.concurrency
    
    @classmethod
    def setUpClass(cls):
        s = server(name=cls.name(), concurrency=cls.concurrency,
                   bind='127.0.0.1:0')
        cls.app = yield send('arbiter', 'run', s)
        cls.uri = 'http://{0}:{1}'.format(*cls.app.address)
        cls.client = HttpClient()
        
    @classmethod
    def tearDownClass(cls):
        if cls.app is not None:
            yield send('arbiter', 'kill_actor', cls.app.name)
    
    @run_on_arbiter
    def testMeta(self):
        app = yield get_application(self.name())
        self.assertEqual(app.name, self.name())
        self.assertTrue(app.monitor.running())
        self.assertEqual(app, app.app)
        self.assertEqual(str(app), app.name)
        self.assertEqual(app.cfg.bind, '127.0.0.1:0')
        
    def testResponse(self):
        c = self.client
        response = yield c.get(self.uri).on_finished
        self.assertEqual(response.status_code, 200)
        content = response.get_content()
        self.assertEqual(content, b'Hello World!\n')
        headers = response.headers
        self.assertTrue(headers)
        self.assertEqual(headers['content-type'], 'text/plain')
        self.assertEqual(headers['server'], SERVER_SOFTWARE)
    
    def testTimeIt(self):
        c = self.client
        response = c.timeit(5, 'get', self.uri)
        #cc = list(c.connection_pools.values())[0]._concurrent_connections
        #self.assertTrue(cc)
        yield response
        self.assertTrue(response.locked_time >= 0)
        self.assertTrue(response.total_time >= response.locked_time)
        self.assertEqual(response.num_failures, 0)
        
    def test_getbench(self):
        c = self.client
        yield MultiDeferred((c.get(self.uri) for _ in range(1))).lock()
    test_getbench.__benchmark__ = True


@dont_run_with_thread
class TestHelloWorldProcess(TestHelloWorldThread):
    concurrency = 'process'
    


HTML_VISITORS = {}


def html_visitor(tag):
    return HTML_VISITORS.get(tag, base)


class HtmlType(type):
    
    def __new__(cls, name, bases, attrs):
        new_class = super(HtmlType,cls).__new__(cls, name, bases, attrs)
        HTML_VISITORS[name.lower()] = new_class()
        return new_class

class Base(object):
    
    def get_form_value(self, html):
        return html.attr('value')
    
    def set_form_value(self, html, value):
        html.attr('value', value)
    

base = Base()
HtmlVisitor = HtmlType('HtmlVisitor', (Base,), {})
    
class Textarea(HtmlVisitor):
    
    def get_form_value(self, html):
        return html.children[0] if html.children else ''
    
    def set_form_value(self, html, value):
        html.remove_all()
        html.append(value)
        
        
class Select(HtmlVisitor):
    
    def get_form_value(self, html):
        values = []
        for child in html.children:
            if child.attr('selected') == 'selected':
                values.append(child.attr('value'))
        if html.attr('multiple') == 'multiple':
            return values
        elif values:
            return values[0]
            
        return html.children[0] if html.children else ''
    
    def set_form_value(self, html, value):
        if html.attr('multiple') == 'multiple':
            for child in html.children:
                if child.attr('value' == value):
                    child.attr('selected', 'selected')
                    break
        else:
            for child in html.children:
                if child.attr('value' == value):
                    child.attr('selected', 'selected')
                else:
                    child._attr.pop('selected', None)
    


import pulsar
from pulsar.utils import events
from pulsar.utils.httpurl import ispy3k
from pulsar.apps import test

try:
    import coverage
except ImportError:
    coverage = None


def stop_coverage(sender, **kwargs):
    cov = pulsar.process_local_data('COVERAGE')
    cov.stop()
    

class Coverage(test.TestPlugin):
    ''':class:`pulsar.apps.test.Plugin` for code coverage.'''
    desc = '''Test coverage with coverage'''
    
    def on_start(self):
        if coverage:
            cov = pulsar.process_local_data('COVERAGE')
            if not cov:
                # START COVERAGE FOR THIS PROCESS
                cov = coverage.coverage()
                pulsar.process_local_data().COVERAGE = cov
                cov.start()
                events.bind('exit', stop_coverage, sender=pulsar.Actor)
                


from collections import deque

from pulsar import send, command
from pulsar.utils.pep import itervalues
from pulsar.apps.tasks import backends, states
from pulsar.apps.pubsub import PubSub


class TaskBackend(backends.TaskBackend):
        
    def put_task(self, task_id):
        return send(self.name, 'put_task', task_id)
    
    def get_task(self, task_id=None, timeout=1):
        return send(self.name, 'get_task', task_id, timeout)
    
    def get_tasks(self, **filters):
        return send(self.name, 'get_tasks', **filters)
        
    def save_task(self, task_id, **params):
        return send(self.name, 'save_task', task_id, **params)
    
    def delete_tasks(self, ids=None):
        return send(self.name, 'delete_tasks', ids)
    
        
class LocalTaskBackend(object):
    
    def __init__(self, name):
        self.pubsub = PubSub(name=name)
        self._tasks = {}
        self.queue = deque()
    
    def put_task(self, task_id):
        if task_id in self._tasks:
            task = self._tasks[task_id]
            task.status = states.QUEUED 
            self.queue.append(task.id)
            return task.id

    def save_task(self, task_id, **params):
        task = self._tasks.get(task_id)
        if task:
            for field, value in params.items():
                setattr(task, field, value)
        else: # create a new task
            task = backends.Task(task_id, **params)
            self._tasks[task.id] = task
        return task.id
    
    def get_task(self, task_id, timeout):        
        if not task_id:
            try:
                task_id = self.queue.popleft()
            except IndexError:
                return None
        return self._tasks.get(task_id)

    def delete_tasks(self, ids):
        ids = ids or list(self._tasks)
        deleted = []
        for id in ids:
            task = self._tasks.pop(id, None)
            if task:
                deleted.append(id)
        return deleted
    
    def get_tasks(self, **filters):
        tasks = []
        if filters:
            fs = []
            for name, value in filters.items():
                if not isinstance(value, (list, tuple, set, frozenset)):
                    value = (value,)
                fs.append((name, value))
            # Loop over tasks
            for task in itervalues(self._tasks):
                select = True
                for name, values in fs:
                    value = getattr(task, name, None)
                    if value not in values:
                        select = False
                        break
                if select:
                    tasks.append(task)
        return tasks
    
#################################################    TASKQUEUE COMMANDS
@command()
def save_task(request, task_id, **params):
    return _get_tasks(request.actor).save_task(task_id, **params)

@command()
def delete_tasks(request, ids):
    return _get_tasks(request.actor).delete_tasks(ids)

@command()
def get_task(request, task_id=None, timeout=1):
    return _get_tasks(request.actor).get_task(task_id, timeout)

@command()
def get_tasks(request, **filters):
    return _get_tasks(request.actor).get_tasks(**filters)

@command()
def put_task(request, task_id):
    return _get_tasks(request.actor).put_task(task_id)

def _get_tasks(actor):
    tasks = actor.params.TASKQUEUE_TASKS
    if tasks is None:
        actor.params.TASKQUEUE_TASKS = tasks = LocalTaskBackend(name=actor.name)
    return tasks


'''
Constats used throughout pulsar.

.. _actor-states:

Actor States
~~~~~~~~~~~~~~

Actor state constants are access via::

    from pulsar import ACTOR_STATES
    
They are:  

* ``ACTOR_STATES.INITIAL = 0`` when an actor is just created, before the
  :class:`pulsar.Actor.start` method is called.
* ``ACTOR_STATES.STARTING = 2`` when :class:`pulsar.Actor.start` method is called.
* ``ACTOR_STATES.RUN = 3`` when :class:`pulsar.Actor.event_loop` is up and running.
* ``ACTOR_STATES.STOPPING = 4`` when :class:`pulsar.Actor.stop` has been called
  for the first time and the actor is running.
  
Constants
~~~~~~~~~~~~~~~~~~~~~~~~

* ``ACTOR_ACTION_TIMEOUT`` used by :class:`pulsar.Monitor` to kill actors
  which don't respond to the stop command.

'''
from pulsar.utils.structures import AttributeDictionary, FrozenDict

# LOW LEVEL CONSTANTS - NO NEED TO CHANGE THOSE ###########################
ACTOR_STATES = AttributeDictionary(INITIAL=0X0,
                                   INACTIVE=0X1,
                                   STARTING=0x2,
                                   RUN=0x3,
                                   STOPPING=0x4,
                                   CLOSE=0x5,
                                   TERMINATE=0x6)
ACTOR_STATES.DESCRIPTION = {ACTOR_STATES.INACTIVE: 'inactive',
                            ACTOR_STATES.INITIAL: 'initial',
                            ACTOR_STATES.STARTING: 'starting',
                            ACTOR_STATES.RUN: 'running',
                            ACTOR_STATES.STOPPING: 'stopping',
                            ACTOR_STATES.CLOSE: 'closed',
                            ACTOR_STATES.TERMINATE:'terminated'}
SPECIAL_ACTORS = ('monitor', 'arbiter')
#
# TIMEOUT FOR WHEN AN ACTOR IS NOT RESPONSING TO A STOP COMMAND
ACTOR_ACTION_TIMEOUT = 5
MIN_NOTIFY = 3     # DON'T NOTIFY BELOW THIS INTERVAL
MAX_NOTIFY = 30    # NOTIFY AT LEAST AFTER THESE SECONDS
ACTOR_TIMEOUT_TOLE = 0.3  # NOTIFY AFTER THIS TIMES THE TIMEOUT
ACTOR_TERMINATE_TIMEOUT = 2 # TIMEOUT WHEN JOINING A TERMINATING ACTOR
#
# SPECIAL objects for Deferred
CONTINUE = object()
NOT_DONE = object()
CLEAR_ERRORS = object()
#
# Globals
EMPTY_TUPLE = ()
EMPTY_DICT = FrozenDict()



'''This module implements several WSGI middleware which does not
serve request but instead perform initialization and sanity checks.
It also introduces the **Response middlewares** implemented as a subclass of 
:class:`ResponseMiddleware`. Response middlewares are used by the
:class:`pulsar.apps.wsgi.wrappers.WsgiResponse` to modify/add headers and
manipulate content.

.. _wsgi-additional-middleware:

Additional WSGI Middlewares
============================

Several :ref:`wsgi middleware <wsgi-middleware>` useful in several applications.

clean path
~~~~~~~~~~~~~~~~~~
.. autofunction:: clean_path_middleware

cookie
~~~~~~~~~~~~~~~~~~
.. autofunction:: cookies_middleware

authorization
~~~~~~~~~~~~~~~~~~
.. autofunction:: authorization_middleware


.. _wsgi-response-middleware:

Response Middlewares
=============================

Response middleware are callable objects which can be used in conjunction
with pulsar :ref:`application handlers <wsgi-handlers>`. They must return
a :ref:`WsgiResponse <wsgi-response>` which can be the same as
the one passed to the callable or a brand new one.

Interface
~~~~~~~~~~~~~~~~~~

.. autoclass:: ResponseMiddleware
   :members:
   :member-order: bysource
   
GZip Middleware
~~~~~~~~~~~~~~~~~~~~~~
.. autoclass:: GZipMiddleware
   :members:
   :member-order: bysource
   
'''
import re
from gzip import GzipFile

import pulsar
from pulsar.utils.httpurl import BytesIO, parse_cookie

from .plugins import parse_authorization_header

re_accepts_gzip = re.compile(r'\bgzip\b')


__all__ = ['clean_path_middleware',
           'AccessControl',
           'GZipMiddleware',
           'cookies_middleware',
           'authorization_middleware',
           'is_streamed']


def is_streamed(content):
    try:
        len(content)
    except TypeError:
        return True
    return False

def clean_path_middleware(environ, start_response):
    '''Clean url from double slashes and redirect if needed.'''
    path = environ['PATH_INFO']
    if '//' in path:
        url = re.sub("/+" , '/', path)
        if not url.startswith('/'):
            url = '/%s' % url
        qs = environ['QUERY_STRING']
        if qs:
            url = '%s?%s' % (url, qs)
        raise pulsar.HttpRedirect(url)

def cookies_middleware(environ, start_response):
    '''Parse the ``HTTP_COOKIE`` key in the ``environ`` and set
the new ``http.environ`` key in ``environ`` with a dictionary of cookies
obtained via the :func:`pulsar.utils.httpurl.parse_cookie` function.'''
    c = environ.get('http.cookie')
    if not isinstance(c, dict):
        c = environ.get('HTTP_COOKIE', '')
        if not c:
            c = {}
        else:
            if not isinstance(c, str):
                c = c.encode('utf-8')
            c = parse_cookie(c)
        environ['http.cookie'] = c

def authorization_middleware(environ, start_response):
    """Parse the ``HTTP_AUTHORIZATION`` key in the ``environ`` if available
and set the ``http.authorization`` key in ``environ`` with the result
obtained from :func:`pulsar.apps.wsgi.plugins.parse_authorization_header`
function."""
    key = 'http.authorization'
    c = environ.get(key)
    if c is None:
        code = 'HTTP_AUTHORIZATION'
        if code in environ:
            environ[key] = parse_authorization_header(environ[code])

#####################################################    RESPONSE MIDDLEWARE
class ResponseMiddleware(object):
    '''Bas class for :class:`pulsar.apps.wsgi.wrappers.WsgiResponse`
middlewares. The focus of this class is the :meth:`execute` method where
the middleware logic is implemented.'''
    def version(self, environ):
        return environ.get('wsgi.version')

    def available(self, environ, response):
        '''Check if this :class:`ResponseMiddleware` can be applied to
the *response* object.

:param environ: a WSGI environ dictionary.
:param response: a :class:`pulsar.apps.wsgi.wrappers.WsgiResponse`
:return: ``True`` or ``False``.'''
        return True

    def __call__(self, environ, response):
        if not self.available(environ, response):
            return response
        resp = self.execute(environ, response)
        return resp if resp is not None else response

    def execute(self, environ, response):
        '''Manipulate *response*, called only if the :meth:`available`
method returns ``True``.'''
        pass


class AccessControl(ResponseMiddleware):
    '''A response middleware which add the ``Access-Control-Allow-Origin``
response header.'''
    def __init__(self, origin='*', methods=None):
        self.origin = origin
        self.methods = methods

    def available(self, environ, response):
        return response.status_code == 200

    def execute(self, environ, response):
        response.headers['Access-Control-Allow-Origin'] = self.origin
        if self.methods:
            response.headers['Access-Control-Allow-Methods'] = self.methods


class GZipMiddleware(ResponseMiddleware):
    """A :class:`ResponseMiddleware` for compressing content if the request
allows gzip compression. It sets the Vary header accordingly.

The compression implementation is from
http://jython.xhaus.com/http-compression-in-python-and-jython
    """
    def __init__(self, min_length=200):
        self.min_length = min_length

    def available(self, environ, response):
        # It's not worth compressing non-OK or really short responses
        if response.status_code == 200 and not response.is_streamed:
            if response.length() < self.min_length:
                return False
            headers = response.headers
            # Avoid gzipping if we've already got a content-encoding.
            if 'Content-Encoding' in headers:
                return False
            # MSIE have issues with gzipped response of various content types.
            if "msie" in environ.get('HTTP_USER_AGENT', '').lower():
                ctype = headers.get('Content-Type', '').lower()
                if not ctype.startswith("text/") or "javascript" in ctype:
                    return False
            ae = environ.get('HTTP_ACCEPT_ENCODING', '')
            if not re_accepts_gzip.search(ae):
                return False
            return True

    def execute(self, environ, response):
        headers = response.headers
        headers.add_header('Vary', 'Accept-Encoding')
        content = b''.join(response.content)
        response.content = (self.compress_string(content),)
        response.headers['Content-Encoding'] = 'gzip'

    def compress_string(self, s):
        zbuf = BytesIO()
        zfile = GzipFile(mode='wb', compresslevel=6, fileobj=zbuf)
        zfile.write(s)
        zfile.close()
        return zbuf.getvalue()



'''Tests the taskqueue local backend.'''
from random import random
from time import time
from datetime import datetime, timedelta

from pulsar import send, get_application, multi_async
from pulsar.apps import tasks, rpc
from pulsar.utils.timeutils import timedelta_seconds
from pulsar.apps.test import unittest, run_on_arbiter, dont_run_with_thread

from .manage import server


CODE_TEST = '''\
import time
def task_function(N = 10, lag = 0.1):
    time.sleep(lag)
    return N*N
'''
            

class TaskQueueBase(object):
    concurrency = 'thread'
    schedule_periodic = True
    # used for both keep-alive and timeout in JsonProxy
    # long enough to allow to wait for tasks
    rpc_timeout = 500
    apps = ()
    
    @classmethod
    def name(cls):
        return cls.__name__.lower()
    
    @classmethod
    def task_backend(cls):
        return None
    
    @classmethod
    def rpc_name(cls):
        return 'rpc_%s' % cls.name()
    
    @classmethod
    def setUpClass(cls):
        # The name of the task queue application
        s = server(name=cls.name(),
                   rpc_bind='127.0.0.1:0',
                   concurrent_tasks=4,
                   concurrency=cls.concurrency,
                   rpc_concurrency=cls.concurrency,
                   rpc_keep_alive=cls.rpc_timeout,
                   task_backend=cls.task_backend(),
                   script=__file__,
                   schedule_periodic=cls.schedule_periodic)
        cls.apps = yield send('arbiter', 'run', s)
        # make sure the time out is high enough (bigger than test-timeout)
        cls.proxy = rpc.JsonProxy('http://%s:%s' % cls.apps[1].address,
                                  timeout=cls.rpc_timeout)

    @classmethod
    def tearDownClass(cls):
        cmnds = [send('arbiter', 'kill_actor', a.name) for a in cls.apps]
        yield multi_async(cmnds)
        
        
class TestTaskQueueOnThread(TaskQueueBase, unittest.TestCase):

    def test_meta(self):
        '''Tests meta attributes of taskqueue'''
        app = yield get_application(self.name())
        self.assertTrue(app)
        self.assertEqual(app.name, self.name())
        self.assertFalse(app.cfg.address)
        self.assertEqual(app.cfg.concurrent_tasks, 4)
        self.assertEqual(app.backend.backlog, 4)
        self.assertTrue(app.backend.registry)
        self.assertEqual(app.cfg.concurrency, self.concurrency)
        backend = app.backend
        self.assertFalse(backend.entries)
        job = app.backend.registry['runpycode']
        self.assertEqual(job.type, 'regular')
        self.assertTrue(job.can_overlap)
        id = job.make_task_id((),{})
        self.assertTrue(id)
        self.assertNotEqual(id, job.make_task_id((),{}))

    def test_pubsub(self):
        '''Tests meta attributes of taskqueue'''
        app = yield get_application(self.name())
        self.assertFalse(app.backend.local.pubsub)
        pubsub = app.backend.pubsub
        self.assertEqual(app.backend.local.pubsub, pubsub)
        # the pubsub name is the same as the task queue application
        self.assertEqual(pubsub.name, app.name)
        if not self.task_backend():
            self.assertEqual(pubsub.backend.connection_string,
                             'local://?name=%s' % app.name)

    def test_rpc_meta(self):
        app = yield get_application(self.rpc_name())
        self.assertTrue(app)
        self.assertEqual(app.name, self.rpc_name())
        self.assertEqual(app.cfg.address, ('127.0.0.1', 0))
        self.assertNotEqual(app.cfg.address, app.address)
        self.assertEqual(app.cfg.concurrency, self.concurrency)
        wsgi_handler = app.callable.handler
        self.assertEqual(len(wsgi_handler.middleware), 1)
        router = wsgi_handler.middleware[0] 
        self.assertTrue(router.post)
        root = router.post
        tq = root.taskqueue
        self.assertEqual(tq, self.name())
        
    def test_registry(self):
        app = yield get_application(self.name())
        self.assertTrue(isinstance(app.backend.registry, dict))
        regular = app.backend.registry.regular()
        periodic = app.backend.registry.periodic()
        self.assertTrue(regular)
        self.assertTrue(periodic)

    def test_run_new_task_asynchronous_from_test(self):
        app = yield get_application(self.name())
        r = yield app.backend.run('asynchronous', lag=3)
        r = yield self.proxy.wait_for_task(r)
        self.assertEqual(r['status'], tasks.SUCCESS)
        result = r['result']
        self.assertTrue(result['loops'])
        self.assertTrue(result['time'] > 3)

    def test_run_new_task_asynchronous(self):
        r = yield self.proxy.run_new_task(jobname='asynchronous', lag=3)
        r = yield self.proxy.wait_for_task(r)
        self.assertEqual(r['status'], tasks.SUCCESS)
        result = r['result']
        self.assertTrue(result['loops'])
        self.assertTrue(result['time'] > 3)
      
    def test_run_new_task_asynchronous_wait_on_test(self):
        app = yield get_application(self.name())
        r = yield self.proxy.run_new_task(jobname='asynchronous', lag=3)
        r = yield app.backend.wait_for_task(r)
        self.assertEqual(r.status, tasks.SUCCESS)
        self.assertTrue(r.result['loops'])
        self.assertTrue(r.result['time'] > 3)
        
    def test_run_new_simple_task_from_test(self):
        app = yield get_application(self.name())
        r = yield app.backend.run('addition', a=1, b=2)
        r = yield self.proxy.wait_for_task(r)
        self.assertEqual(r['status'], tasks.SUCCESS)
        self.assertEqual(r['result'], 3)
    
    def test_run_new_simple_task(self):
        r = yield self.proxy.run_new_task(jobname='addition', a=40, b=50)
        r = yield self.proxy.wait_for_task(r)
        self.assertEqual(r['status'], tasks.SUCCESS)
        self.assertEqual(r['result'], 90)
    
    def test_not_overlap(self):
        sec = 2 + random()
        app = yield get_application(self.name())
        self.assertEqual(app.name, app.backend.name)
        self.assertTrue('notoverlap' in app.backend.registry)
        r1 = yield app.backend.run('notoverlap', sec)
        self.assertTrue(r1)
        r2 = yield app.backend.run('notoverlap', sec)
        self.assertFalse(r2)
        # We need to make sure the first task is completed
        r1 = yield app.backend.wait_for_task(r1)
        self.assertEqual(r1.status, tasks.SUCCESS)
        self.assertTrue(r1.result > sec)

    def test_run_new_task_error(self):
        yield self.async.assertRaises(rpc.InvalidParams,
                            self.proxy.run_new_task())
        yield self.async.assertRaises(rpc.InternalError,
                            self.proxy.run_new_task(jobname='xxxx', bla='foo'))

    def test_run_new_task_run_py_code(self):
        '''Run a new task from the *runpycode* task factory.'''
        r = yield self.proxy.run_new_task(jobname='runpycode', code=CODE_TEST, N=3)
        self.assertTrue(r)
        r = yield self.proxy.wait_for_task(r)
        self.assertEqual(r['status'], tasks.SUCCESS)
        self.assertEqual(r['result'], 9)

    def test_run_new_task_periodicerror(self):
        r = yield self.proxy.run_new_task(jobname='testperiodicerror')
        r = yield self.proxy.wait_for_task(r)
        self.assertEqual(r['status'], tasks.FAILURE)
        self.assertTrue('kaputt' in r['result'])
     
    def test_run_new_task_expiry(self):
        r = yield self.proxy.run_new_task(jobname='addition', a=40, b=50,
                                          meta_data={'expiry': time()})
        self.assertTrue(r)
        r = yield self.proxy.wait_for_task(r)
        self.assertEqual(r['status'], tasks.REVOKED)
           
    def test_delete_task(self):
        app = yield get_application(self.name())
        id = yield app.backend.run('addition', 1, 4)
        r1 = yield app.backend.wait_for_task(id)
        self.assertEqual(r1.result, 5)
        deleted = yield app.backend.delete_tasks([r1.id, 'kjhbkjb'])
        self.assertEqual(len(deleted), 1)
        r1 = yield app.backend.get_task(r1.id)
        self.assertFalse(r1)
      
    def __test_run_producerconsumer(self):
        '''A task which produce other tasks'''
        sample = 10
        r = yield self.proxy.run_new_task(jobname='standarddeviation',
                                          sample=sample, size=100)
        self.assertTrue(r)
        r = yield self.proxy.wait_for_task(r)
        self.assertEqual(r['status'], tasks.SUCCESS)
        self.assertTrue(tasks.nice_task_message(r))
        # We check for the tasks created
        stasks = []
        while len(stasks) < sample:
            ts = yield self.proxy.get_tasks(from_task=r['id'])
            stasks = []
            for t in ts:
                if t['status'] not in tasks.UNREADY_STATES:
                    stasks.append(t)
                    self.assertEqual(t['status'], tasks.SUCCESS)
        self.assertEqual(len(stasks), sample)

    ##    RPC TESTS
    def test_rpc_job_list(self):
        jobs = yield self.proxy.job_list()
        self.assertTrue(jobs)
        self.assertTrue(isinstance(jobs, list))
        d = dict(jobs)
        pycode = d['runpycode']
        self.assertEqual(pycode['type'], 'regular')

    def test_check_next_run(self):
        app = yield get_application(self.name())
        backend = app.backend
        now = datetime.now()
        backend.tick()
        #self.assertTrue(backend.next_run > now)
        
    def test_rpc_ping(self):
        yield self.async.assertEqual(self.proxy.ping(), 'pong')

    def test_rpc_job_list(self):
        jobs = yield self.proxy.job_list()
        self.assertTrue(jobs)
        self.assertTrue(isinstance(jobs, list))
        d = dict(jobs)
        pycode = d['runpycode']
        self.assertEqual(pycode['type'], 'regular')
      
    def test_rpc_job_list_with_names(self):
        jobs = yield self.proxy.job_list(jobnames=['runpycode'])
        self.assertEqual(len(jobs), 1)
        jobs = yield self.proxy.job_list(jobnames=['xxxxxx'])
        self.assertEqual(len(jobs), 0)
       
    def test_rpc_next_scheduled_tasks(self):
        next = yield self.proxy.next_scheduled_tasks()
        self.assertTrue(next)
        self.assertEqual(len(next), 2)
        next = yield self.proxy.next_scheduled_tasks(jobnames=['testperiodic'])
        self.assertTrue(next)
        self.assertEqual(len(next), 2)
        self.assertEqual(next[0], 'testperiodic')
        self.assertTrue(next[1] >= 0)
        
    def test_id_not_overlap(self):
        '''Check `make_task_id` when `can_overlap` attribute is set to False.'''
        app = yield get_application(self.name())
        job = app.backend.registry['notoverlap']
        self.assertEqual(job.type, 'regular')
        self.assertFalse(job.can_overlap)
        #
        id = job.make_task_id((),{})
        self.assertTrue(id)
        self.assertEqual(id, job.make_task_id((),{}))
        #
        id = job.make_task_id((10,'bla'),{'p':45})
        self.assertTrue(id)
        self.assertEqual(id,job.make_task_id((10,'bla'),{'p':45}))
        #
        id = job.make_task_id((),{'p':45,'c':'bla'})
        self.assertTrue(id)
        self.assertEqual(id,job.make_task_id((),{'p':45,'c':'bla'}))
        self.assertNotEqual(id,job.make_task_id((),{'p':45,'d':'bla'}))
        self.assertNotEqual(id,job.make_task_id((),{'p':45,'c':'blas'}))


@dont_run_with_thread
class TestTaskQueueOnProcess(TestTaskQueueOnThread):
    concurrency = 'process'



import inspect

__all__ = ['checkarity']


def checkarity(func, args, kwargs, discount=0):
    '''Check if arguments respect a given function arity and return
a error message if the check did not pass, otherwise it returns ``None``.

:parameter func: the function.
:parameter args: function arguments.
:parameter kwargs: function key-valued parameters.
:parameter discount: optional integer which discount the number of
                     positional argument to check. Default ``0``.'''
    spec = inspect.getargspec(func)
    args = list(args)
    defaults = list(spec.defaults or ())
    len_defaults = len(defaults)
    len_args = len(spec.args) - discount
    len_args_input = len(args)
    minlen = len_args - len_defaults
    totlen = len_args_input + len(kwargs)
    maxlen = len_args
    if spec.varargs or spec.keywords:
        maxlen = None
        if not minlen:
            return
    
    if not spec.defaults and maxlen:
        start = '"{0}" takes'.format(func.__name__)
    else:
        if maxlen and totlen > maxlen:
            start = '"{0}" takes at most'.format(func.__name__)
        else:
            start = '"{0}" takes at least'.format(func.__name__)
        
    if totlen < minlen:
        return '{0} {1} parameters. {2} given.'.format(start,minlen,totlen)
    elif maxlen and totlen > maxlen:
        return '{0} {1} parameters. {2} given.'.format(start,maxlen,totlen)
    
    # Length of parameter OK, check names
    if len_args_input < len_args:
        l = minlen - len_args_input
        for arg in spec.args[discount:]:
            if args:
                args.pop(0)
            else:
                if l>0:
                    if defaults:
                        defaults.pop(0)
                    elif arg not in kwargs:
                        return '"{0}" has missing "{1}" parameter.'\
                                    .format(func.__name__,arg)
                kwargs.pop(arg,None)
            l -= 1
        if kwargs and maxlen:
            s = ''
            if len(kwargs) > 1:
                s = 's'
            p = ', '.join('"{0}"'.format(p) for p in kwargs)
            return '"{0}" does not accept {1} parameter{2}.'\
                            .format(func.__name__,p,s)
    elif len_args_input > len_args + len_defaults:
        n = len_args + len_defaults
        start = '"{0}" takes'.format(func.__name__)
        return '{0} {1} positional parameters. {2} given.'\
                        .format(start,n,len_args_input)

                
        
        
        


'''Tests django chat application.'''
import json

from pulsar import send, get_application, Deferred
from pulsar.utils.path import Path
from pulsar.apps import wsgi, http, ws
from pulsar.apps.test import unittest, dont_run_with_thread

try:
    manage = Path(__file__).add2python('manage', up=1)
except ImportError:
    manage = None


def start_server(actor, name, argv):
    # we need to make sure djangoapp is in the python path
    actor.params.django_pulsar_name = name
    manage.execute_from_command_line(argv)
    app = yield get_application(name)
    yield app.event('start')


class MessageHandler(ws.WS):
    
    def __init__(self):
        self.new_future()
        
    def new_future(self):
        self.future = Deferred()
        
    def on_message(self, request, message):
        future = self.future
        self.new_future()
        future.callback(message)


@unittest.skipUnless(manage, 'Requires django')
class TestDjangoChat(unittest.TestCase):
    concurrency = 'thread'
    app = None
    
    @classmethod
    def setUpClass(cls):
        name = 'django_' + cls.concurrency
        argv = [__file__, 'pulse', '--bind', '127.0.0.1:0',
                '--concurrency', cls.concurrency]
        cls.app = yield send('arbiter', 'run', start_server, name, argv)
        cls.uri = 'http://{0}:{1}'.format(*cls.app.address)
        cls.ws = 'ws://{0}:{1}/message'.format(*cls.app.address)
        cls.http = http.HttpClient()
        
    @classmethod
    def tearDownClass(cls):
        if cls.app:
            return send('arbiter', 'kill_actor', cls.app.name)

    def test_home(self):
        result = yield self.http.get(self.uri).on_finished
        self.assertEqual(result.status_code, 200)
        
    def test_404(self):
        result = yield self.http.get('%s/bsjdhcbjsdh' % self.uri).on_finished
        self.assertEqual(result.status_code, 404)
        
    def test_handshake(self):
        ws = yield self.http.get(self.ws).on_finished
        response = ws.handshake 
        self.assertEqual(response.status_code, 101)
        self.assertEqual(response.headers['upgrade'], 'websocket')
        self.assertEqual(response.connection, None)
        self.assertTrue(ws.connection)
        
    def test_websocket(self):
        ws = yield self.http.get(self.ws).on_finished
        self.assertTrue(ws)
        ws.handler = MessageHandler()
        ws.write('Hello there!')
        data = yield ws.handler.future
        data = json.loads(data)
        self.assertEqual(data['message'], 'Hello there!')
        
        
@dont_run_with_thread
class TestDjangoChat_Process(TestDjangoChat):
    concurrency = 'process'
    


import sys
import os
import atexit
import logging
from time import time
import random
from functools import partial

from pulsar import AlreadyRegistered, HaltServer, CommandError,\
                   ActorAlreadyStarted, system, Config, platform
from pulsar.utils.pep import pickle, set_event_loop_policy, itervalues
from pulsar.utils.log import LogginMixin

from .eventloop import EventLoop, setid, signal
from .defer import Deferred, EventHandler, log_failure
from .threads import Empty, ThreadQueue, ThreadPool
from .proxy import ActorProxy, get_proxy, ActorProxyMonitor, ActorIdentity
from .mailbox import MailboxClient, command_in_context
from .access import set_actor, is_mainthread, get_actor, remove_actor
from .consts import *


__all__ = ['is_actor', 'send', 'Actor', 'ACTOR_STATES', 'Pulsar', 'ThreadQueue']


def is_actor(obj):
    return isinstance(obj, Actor)

def send(target, action, *args, **params):
    '''Send a :ref:`message <api-remote_commands>` to *target* to perform
a given *action*.

:parameter target: the :class:`Actor` id or an :class:`ActorProxy` or name of
    the target actor which will receive the message.
:parameter action: the name of the :ref:`remote command <api-remote_commands>`
    to perform in the *target* :class:`Actor`.
:parameter args: positional arguments to pass to the
    :ref:`remote command <api-remote_commands>` *action*.
:parameter params: dictionary of parameters to pass to
    :ref:`remote command <api-remote_commands>` *action*.
:rtype: a :class:`Deferred` if the action acknowledge the caller or `None`.

Typical example::

    >>> a = spawn()
    >>> r = a.add_callback(lambda p: send(p,'ping'))
    >>> r.result
    'pong'
'''
    return get_actor().send(target, action, *args, **params)

def stop_on_error(f):
    def _(self, *args, **kwargs):
        try:
            return f(self, *args, **kwargs)
        except Exception as e:
            self.stop(e)
    return _


class Pulsar(LogginMixin):
    
    def configure_logging(self, **kwargs):
        configure_logging = super(Pulsar, self).configure_logging
        configure_logging(logger='pulsar',
                          config=self.cfg.logconfig,
                          level=self.cfg.loglevel,
                          handlers=self.cfg.loghandlers) 
        configure_logging(logger='pulsar.%s' % self.name,
                          config=self.cfg.logconfig,
                          level=self.cfg.loglevel,
                          handlers=self.cfg.loghandlers)
        

class Actor(EventHandler, Pulsar, ActorIdentity):
    '''The base class for parallel execution in pulsar. In computer science,
the **Actor model** is a mathematical model of concurrent computation that
treats *actors* as the universal primitives of computation.
In response to a message that it receives, an actor can make local decisions,
create more actors, send more messages, and determine how to respond to
the next message received.

The current implementation allows for actors to perform specific tasks such
as listening to a socket, acting as http server, consuming
a task queue and so forth.

To spawn a new actor::

    >>> from pulsar import spawn
    >>> a = spawn()
    >>> a.is_alive()
    True

Here ``a`` is actually a reference to the remote actor, it is
an :class:`ActorProxy`.

**ATTRIBUTES**

.. attribute:: name

    The name of this :class:`Actor`.
    
.. attribute:: aid

    Unique ID for this :class:`Actor`.
    
.. attribute:: impl

    The :class:`Concurrency` implementation for this :class:`Actor`.
    
.. attribute:: event_loop

    An instance of :class:`EventLoop` which listen for input/output events
    on sockets or socket-like :class:`Transport`. It is the driver of the
    :class:`Actor`. If the :attr:event_loop` stps, the :class:`Actor` stop
    running and goes out of scope.

.. attribute:: mailbox

    Used to send and receive :ref:`actor messages <tutorials-messages>`.
    
.. attribute:: address

    The socket address for this :attr:`Actor.mailbox`.
    
.. attribute:: proxy

    Instance of a :class:`ActorProxy` holding a reference
    to this :class:`Actor`. The proxy is a lightweight representation
    of the actor which can be shared across different processes
    (i.e. it is pickable).

.. attribute:: state

    The actor :ref:`numeric state <actor-states>`.
    
.. attribute:: thread_pool

    A :class:`ThreadPool` associated with this :class:`Actor`. This attribute
    is ``None`` unless one is created via the :meth:`create_thread_pool`
    method.

.. attribute:: params

    A :class:`pulsar.utils.structures.AttributeDictionary` which contains
    parameters which are passed to actors spawned by this actor.
    
.. attribute:: info_state

    Current state description string. One of ``initial``, ``running``,
    ``stopping``, ``closed`` and ``terminated``.
 
**PUBLIC METHODS**
'''
    ONE_TIME_EVENTS = ('start', 'stopping', 'stop')
    exit_code = None
    mailbox = None
    signal_queue = None
    
    def __init__(self, impl):
        super(Actor, self).__init__()
        self.state = ACTOR_STATES.INITIAL
        self._thread_pool = None
        self.__impl = impl
        for name in self.all_events():
            hook = impl.params.pop(name, None)
            if hook:
                self.bind_event(name, hook)
        self.monitor = impl.params.pop('monitor', None)
        self.params = AttributeDictionary(**impl.params)
        self.servers = {}
        del impl.params
        setid(self)
        try:
            self.cfg.post_fork(self)
        except Exception:
            pass

    def __repr__(self):
        return self.impl.unique_name
    
    def __str__(self):
        return self.__repr__()
    
    ############################################################### PROPERTIES
    @property
    def name(self):
        return self.__impl.name

    @property
    def aid(self):
        return self.__impl.aid
    
    @property
    def impl(self):
        return self.__impl
    
    @property
    def cfg(self):
        return self.__impl.cfg
    
    @property
    def proxy(self):
        return ActorProxy(self)

    @property
    def address(self):
        return self.mailbox.address

    @property
    def event_loop(self):
        return self.mailbox.event_loop
    
    @property
    def thread_pool(self):
        return self._thread_pool

    @property
    def info_state(self):
        return ACTOR_STATES.DESCRIPTION[self.state]

    ############################################################################
    ##    HIGH LEVEL API METHODS
    ############################################################################
    def start(self):
        '''Called after forking to start the actor's life. This is where
logging is configured, the :attr:`Actor.mailbox` is registered and the
:attr:`Actor.event_loop` is initialised and started. Calling this method
more than once does nothing.'''
        if self.state == ACTOR_STATES.INITIAL:
            self._started = time() 
            self.configure_logging()
            self._setup_event_loop()
            self.state = ACTOR_STATES.STARTING
            self._run()

    def send(self, target, action, *args, **params):
        '''Send a message to *target* to perform *action* with given
parameters *params*.'''
        target = self.monitor if target == 'monitor' else target
        mailbox = self.mailbox
        if isinstance(target, ActorProxyMonitor):
            mailbox = target.mailbox
        else:
            actor = self.get_actor(target)
            if isinstance(actor, Actor):
                # this occur when sending a message from arbiter to monitors or
                # viceversa.
                return command_in_context(action, self, actor, args, params)
            elif isinstance(actor, ActorProxyMonitor):
                mailbox = actor.mailbox
        if hasattr(mailbox, 'request'):
            return mailbox.request(action, self, target, args, params)
        else:
            raise CommandError('Cannot execute "%s" in %s. Unknown actor '
                               '%s.' % (action, self, target))
    
    def io_poller(self):
        '''Return a IO poller instance which sets the :class:`EventLoop.io`
handler. By default it return nothing so that the best handler for the
system is chosen.'''
        return None
    
    def spawn(self, **params):
        raise RuntimeError('Cannot spawn an actor from an actor.')
    
    def stop(self, exc=None):
        '''Stop the actor by closing its :attr:`mailbox` which
in turns stops the :attr:`event_loop`. Once everything is closed
properly this actor will go out of scope.

:param exc: optional exception.
'''
        log_failure(exc)
        if self.state <= ACTOR_STATES.RUN:
            # The actor has not started the stopping process. Starts it now.
            self.exit_code = getattr(exc, 'exit_code', 1) if exc else 0
            self.state = ACTOR_STATES.STOPPING
            self.fire_event('stopping')
            self.close_thread_pool()
            self._stop()
        elif self.stopped():
            # The actor has finished the stopping process.
            #Remove itself from the actors dictionary
            remove_actor(self)
            self.fire_event('stop')
        return self.event('stop')
    
    def create_thread_pool(self, workers=1):
        '''Create a :class:`ThreadPool` for this :class:`Actor`
if not already present.

:param workers: number of threads to use in the :class:`ThreadPool`
'''
        if self._thread_pool is None:
            self._thread_pool = ThreadPool(processes=workers)
        return self._thread_pool
    
    def close_thread_pool(self):
        '''Close the :attr:`thread_pool`.'''
        if self._thread_pool:
            self._thread_pool.close()
    ###############################################################  STATES
    def running(self):
        '''``True`` if actor is running, that is when the :attr:`state`
is equal to :ref:`ACTOR_STATES.RUN <actor-states>`.'''
        return self.state == ACTOR_STATES.RUN
    
    def active(self):
        '''``True`` if actor is active by being both running and having
the :attr:`event_loop` running.'''
        return self.running()
    
    def started(self):
        '''``True`` if actor has started. It does not necessarily
mean it is running. Its state is greater or equal
:ref:`ACTOR_STATES.RUN <actor-states>`.'''
        return self.state >= ACTOR_STATES.RUN

    def closed(self):
        '''``True`` if actor has exited in an clean fashion. It :attr:`state`
is equal to :ref:`ACTOR_STATES.CLOSE <actor-states>`.'''
        return self.state == ACTOR_STATES.CLOSE

    def stopped(self):
        '''``True`` if actor has exited so that it :attr:`state` is greater
or equal to :ref:`ACTOR_STATES.CLOSE <actor-states>`.'''
        return self.state >= ACTOR_STATES.CLOSE

    def is_arbiter(self):
        '''Return ``True`` if ``self`` is the :class:`Arbiter`.'''
        return False

    def is_monitor(self):
        '''Return ``True`` if ``self`` is a :class:`Monitor`.'''
        return False

    def is_process(self):
        '''boolean indicating if this is an actor on a child process.'''
        return self.impl.kind == 'process'

    def __reduce__(self):
        raise pickle.PicklingError('{0} - Cannot pickle Actor instances'\
                                   .format(self))
    
    ############################################################################
    #    INTERNALS
    ############################################################################
    def _stop(self):
        '''Exit from the :class:`Actor` domain.'''
        self.bind_event('stop', self._bye)
        self.state = ACTOR_STATES.CLOSE
        self.mailbox.close()
    
    def _bye(self, r):
        self.logger.debug('Bye from "%s"', self)
        return r
    
    def periodic_task(self):
        if self.can_continue():
            if self.running():
                self.logger.debug('%s notifying the monitor', self)
                # if an error occurs, shut down the actor
                try:
                    r = self.send('monitor', 'notify', self.info())\
                            .add_errback(self.stop)
                except Exception as e:
                    self.stop(e)
                else:
                    secs = max(ACTOR_TIMEOUT_TOLE*self.cfg.timeout, MIN_NOTIFY)
                    next = min(secs, MAX_NOTIFY)
                    self.event_loop.call_later(next, self.periodic_task)
                    return r
            else:
                self.event_loop.call_soon_threadsafe(self.periodic_task)
        
    @stop_on_error
    def _got_notified(self, result):
        self.logger.info('%s started', self)
        self.fire_event('start')
        
    @stop_on_error
    def hand_shake(self):
        a = get_actor()
        if a is not self:
            set_actor(self)
        self.state = ACTOR_STATES.RUN
        r = self.periodic_task()
        if r:
            r.add_callback(self._got_notified)
        else:
            self.stop()
    
    def get_actor(self, aid):
        '''Given an actor unique id return the actor proxy.'''
        if aid == self.aid:
            return self
        elif aid == 'monitor':
            return self.monitor or self

    def info(self):
        '''Return a dictionary of information related to the actor
status and performance.'''
        if not self.started():
            return
        isp = self.is_process()
        actor = {'name': self.name,
                 'state': self.info_state,
                 'actor_id': self.aid,
                 'uptime': time() - self._started,
                 'thread_id': self.tid,
                 'process_id': self.pid,
                 'is_process': isp,
                 'age': self.impl.age}
        events = {'callbacks': len(self.event_loop._callbacks),
                  'io_loops': self.event_loop.num_loops}
        data = {'actor': actor, 'events': events}
        if isp:
            data['system'] = system.system_info(self.pid)
        return data

    def _setup_event_loop(self):
        # Internal function called at the start of the actor. It builds the
        # event loop which will consume events on file descriptors
        # Build the mailbox first so that when the mailbox closes, it shut
        # down the eventloop.
        event_loop = EventLoop(io=self.io_poller(), logger=self.logger,
                               poll_timeout=self.params.poll_timeout)
        self.mailbox = self._mailbox(event_loop)
        # Inject self as the actor of this thread
        set_actor(self)
        if self.is_process():
            random.seed()
            proc_name = "%s-%s" % (self.cfg.proc_name, self)
            if system.set_proctitle(proc_name):
                self.logger.debug('Set process title to %s', proc_name)
            #system.set_owner_process(cfg.uid, cfg.gid)
            if is_mainthread() and signal and not platform.is_windows:
                self.logger.debug('Installing signals')
                self.signal_queue = ThreadQueue()
                for name in system.ALL_SIGNALS:
                    sig = getattr(signal, 'SIG%s' % name)
                    try:
                        handler = partial(self.signal_queue.put, sig)
                        self.event_loop.add_signal_handler(sig, handler)
                    except ValueError:
                        break
        
    def can_continue(self):
        if self.signal_queue is not None:
            while True:
                try:
                    sig = self.signal_queue.get(timeout=0.01)
                except (Empty, IOError):
                    break
                if sig not in system.SIG_NAMES:
                    self.logger.info("Ignoring unknown signal: %s", sig)
                else:
                    signame = system.SIG_NAMES.get(sig)
                    if sig in system.EXIT_SIGNALS:
                        self.logger.warning("Got signal %s. Stopping.", signame)
                        self.stop()
                        return False
                    else:
                        self.logger.debug('No handler for signal %s.', signame)
        return True
    
    def _run(self, initial=True):
        if initial:
            try:
                self.cfg.when_ready(self)
            except Exception:
                pass
        exc = None
        try:
            self.event_loop.run()
        except Exception as e:
            exc = e
        except HaltServer as e:
            exc = e
            if e.exit_code:
                log_failure(e, msg=str(e), level='critical')
            else:
                log_failure(e, msg='Exiting server.', level='info')
        finally:
            self.stop(exc)
        
    def _mailbox(self, event_loop):
        client = MailboxClient(self.monitor.address, self, event_loop)
        client.event_loop.call_soon_threadsafe(self.hand_shake)
        return client



import io
import sys
import socket
import logging
from inspect import isgenerator
from collections import deque

from pulsar.utils import sockets
from pulsar.utils.pep import get_event_loop, range
from pulsar.utils.sockets import WRITE_BUFFER_MAX_SIZE, get_transport_type,\
                                 create_socket
from pulsar.async.defer import Deferred

LOGGER = logging.getLogger('pulsar.transport')


__all__ = ['Transport', 'SocketTransport', 'TransportProxy', 'create_transport']


def create_transport(protocol, sock=None, address=None, event_loop=None, 
                     source_address=None, **kw):
    '''Create a new connection with a remote server. It returns
a :class:`Transport` for the connection.'''
    sock = create_socket(sock=sock, address=address)
    sock.settimeout(0)
    if source_address:
        sock.bind(source_address)
    transport_factory = get_transport_type(sock.TYPE).transport
    event_loop = event_loop or get_event_loop()
    return transport_factory(event_loop, sock, protocol, **kw)


class TransportType(sockets.TransportType):
    '''A simple metaclass for Servers.'''
    def __new__(cls, name, bases, attrs):
        new_class = super(TransportType, cls).__new__(cls, name, bases, attrs)
        type = getattr(new_class, 'TYPE', None)
        if type is not None:
            cls.TRANSPORT_TYPES[type].transport = new_class
        return new_class


class Transport(TransportType('TransportBase', (), {})):
    '''Base class for pulsar transports. Design to conform with pep-3156_ as
close as possible until it is finalised. A transport is an abstraction on top
of a socket or something similar.
Form pep-3153_:

Transports talk to two things: the other side of the
connection on one hand, and a :attr:`protocol` on the other. It's a bridge
between the specific underlying transfer mechanism and the protocol.
Its job can be described as allowing the protocol to just send and
receive bytes, taking care of all of the magic that needs to happen to those
bytes to be eventually sent across the wire.

The primary feature of a transport is sending bytes to a protocol and receiving
bytes from the underlying protocol. Writing to the transport is done using
the :meth:`write` and :meth:`writelines` methods. The latter method is a
performance optimisation, to allow software to take advantage of specific
capabilities in some transport mechanisms.

.. attribute:: event_loop

    The :class:`EventLoop` for this :class:`Transport`.
    
.. attribute:: protocol

    The :class:`Protocol` for this :class:`Transport`.
    
.. attribute:: sock

    the socket/pipe for this :class:`Transport`.

.. attribute:: connecting

    ``True`` if the transport is connecting with remote server.
    
.. attribute:: writing

    ``True`` if the transport has data in its writing buffer.

.. attribute:: closing

    ``True`` if the transport is about to close.
        
.. attribute:: closed

    ``True`` if the transport is closed.
'''
    closed = False
    def write(self, data):
        '''Write some data bytes to the transport.
        This does not block; it buffers the data and arranges for it
        to be sent out asynchronously.'''
        raise NotImplementedMethod
    
    def writelines(self, list_of_data):
        """Write a list (or any iterable) of data bytes to the transport.
If *list_of_data* is a **generator**, and during iteration an empty byte is
yielded, the function will postpone writing the remaining of the generator
at the next loop in the :attr:`eventloop`."""
        for data in list_of_data:
            self.write(data)
    
    def pause(self):
        """A :class:`Transport` can be paused and resumed. Invoking this
method will cause the transport to buffer data coming from protocols but not
sending it to the :attr:`protocol`. In other words, no data will be passed to
the :meth:`Protocol.data_received` method until :meth:`resume` is called.
        """
        raise NotImplementedError

    def resume(self):
        """Resume the receiving end. Data received will once again be
passed to the :meth:`Protocol.data_received` method."""
        raise NotImplementedError
    
    def close(self, async=True, exc=None):
        """Closes the transport.

        Buffered data will be flushed asynchronously.  No more data
        will be received.  After all buffered data is flushed, the
        protocol's connection_lost() method will (eventually) called
        with None as its argument.
        """
        raise NotImplementedError
    
    def abort(self, exc=None):
        """Closes the transport immediately.

        Buffered data will be lost.  No more data will be received.
        The protocol's connection_lost() method will (eventually) be
        called with None as its argument.
        """
        self.close(async=False, exc=exc)
        
    
class SocketTransport(Transport):
    '''A class:`Transport` based on sockets. It handles events implemented by
the :class:`EventHandler`.
    
.. attribute:: connecting

    The transport is connecting, al writes are buffered until connection
    is established.
    
.. attribute:: writing

    The transport has data in the write buffer
    
.. attribute:: closing

    The transport is about to close.
    
**One Time Events**

* **connection_made** fired when a connections made (for client connections)
* **closing** when transport is about to close
* **connection_lost** fired when a the connections with end-point is lost

**Many Times Events**

* **data_received** fired when new data has arrived
'''
    def __init__(self, event_loop, sock, protocol, max_buffer_size=None,
                  read_chunk_size=None):
        self._protocol = protocol
        self._sock = sock
        self._event_loop = event_loop
        self._closing = False
        self._paused = False
        self._connector = None
        self._read_chunk_size = read_chunk_size or io.DEFAULT_BUFFER_SIZE
        self._read_buffer = []
        self._write_buffer = deque()
    
    def __repr__(self):
        try:
            return self._sock.__repr__()
        except Exception:
            return '<closed>'
    
    def __str__(self):
        return self.__repr__()
    
    @property
    def connecting(self):
        return self._connector is not None
    
    @property
    def writing(self):
        return self._sock is not None and bool(self._write_buffer)
    
    @property
    def closing(self):
        return bool(self._closing)
    
    @property
    def closed(self):
        return self._sock is None
    
    @property
    def sock(self):
        return self._sock
    
    @property
    def address(self):
        if self._sock:
            return self._sock.address
    
    @property
    def event_loop(self):
        return self._event_loop
    
    def fileno(self):
        if self._sock:
            return self._sock.fileno()

    def write(self, data):
        self._check_closed()
        writing = self.writing
        if data:
            assert isinstance(data, bytes)
            if len(data) > WRITE_BUFFER_MAX_SIZE:
                for i in range(0, len(data), WRITE_BUFFER_MAX_SIZE):
                    self._write_buffer.append(data[i:i+WRITE_BUFFER_MAX_SIZE])
            else:
                self._write_buffer.append(data)
        # Try to write only when not waiting for write callbacks
        if not self.connecting and not writing:
            self._do_write()
    
    def pause(self):
        if not self._paused:
            self._paused = True

    def resume(self):
        if self._paused:
            self._paused = False
            buffer = self._read_buffer
            self._read_buffer = []
            for data in buffer:
                self._data_received(chunk)
    
    def close(self, async=True, exc=None):
        if not self.closing:
            self._closing = True
            self._event_loop.remove_reader(self._sock.fileno())
            if async and not self.writing:
                self._event_loop.call_soon(self._shutdown, exc)
        if not async:
            self._shutdown(exc)
    
    ############################################################################
    ###    PULSAR TRANSPORT METHODS.
    def connect(self, address, timeout=None):
        '''Connect this :class:`Transport` to a remote server and
returns ``self``.'''
        self._connector = c = Connector(self, timeout=timeout)
        self.add_reader()
        self.add_connector()
        try:
            if self._protocol_connect(address):
                self.callback_connector(self)
        except Exception:
            self.callback_connector(sys.exc_info())
        return c
        
    def add_listener(self):
        '''Called by :class:`Server`'''
        self._event_loop.add_reader(self.fileno(), self._protocol_accept)
        
    def add_reader(self):
        '''Add reader to the event loop. An optional *error_handler*
can be passed. The error handler will be called once an exception occurs
during asynchronous reading, with the exception instance
as only attribute.'''
        self._event_loop.add_reader(self.fileno(), self._ready_read)
        
    def add_writer(self):
        '''Add writer to the event loop'''
        self._event_loop.add_writer(self.fileno(), self._ready_write)
        
    def add_connector(self):
        '''Add writer to the event loop'''
        self._event_loop.add_connector(self.fileno(), self.callback_connector)
    
    def remove_writer(self):
        self._event_loop.remove_writer(self.fileno())
        
    def callback_connector(self, result=None):
        c = self._connector
        if c:
            self._connector = None
            self._event_loop.remove_connector(self.fileno())
            return c.callback(result)
        
    def is_stale(self):
        if self._sock:
            return sockets.is_closed(self._sock)
        else:
            return True
        
    ############################################################################
    ###    INTERNALS
    def _data_received(self, data):
        if self._paused:
            self._read_buffer.append(data)
        else:
            self._protocol.data_received(data)
                 
    def _check_closed(self):
        if self.closed:
            raise IOError("Transport is closed")
        elif self._closing:
            raise IOError("Transport is closing")
        
    def _shutdown(self, exc=None):
        if self._sock is not None:
            self._write_buffer = deque()
            self._event_loop.remove_writer(self.fileno())
            self._sock.close()
            self._sock = None
            if self.connecting:
                if not exc:
                    try:
                        raise RuntimeError('Could not connect. Closing.')
                    except RuntimeError:
                        exc = sys.exc_info()
                self._connector.callback(exc)
            else:
                self._protocol.connection_lost(exc)
        
    def _ready_read(self):
        # Read from the socket until we get EWOULDBLOCK or equivalent.
        # If any other error occur, abort the connection and re-raise.
        passes = 0
        chunk = True
        while chunk:
            try:
                chunk = self._protocol_read()
                if chunk:
                    self._data_received(chunk)
                elif not passes and chunk == b'':
                    # We got empty data. Close the socket
                    self.close()
                passes += 1
            except Exception:
                self.abort(exc=sys.exc_info())
                raise
                
    def _ready_write(self):
        if self.writing:
            try:
                self._protocol_write()
            except socket.error as e:
                # error during a write, abort
                self.abort(exc=e)
                raise
        if not self.writing:
            self.remove_writer()
            if self._closing:
                self._event_loop.call_soon(self._shutdown)
        
    def _do_write(self):
        if self.writing:
            try:
                self._protocol_write()
            except socket.error:
                self.abort(exc=sys.exc_info())
                raise
        if self.writing:
            self.add_writer()
        elif self._closing:
            self._event_loop.call_soon(self._shutdown)
    
    ############################################################################
    ##    PURE VIRTUALS
    def _protocol_connect(self):
        raise NotImplementedError
    
    def _protocol_read(self):
        raise NotImplementedError
    
    def _protocol_write(self):
        raise NotImplementedError
    
    def _protocol_accept(self):
        raise NotImplementedError
            

class TransportProxy(object):
    '''Provides :class:`Transport` like methods and attributes.'''
    _transport = None
    
    def __repr__(self):
        if self._transport:
            return self._transport.__repr__()
        else:
            return '<closed>'
        
    def __str__(self):
        return self.__repr__()
    
    @property
    def transport(self):
        return self._transport
    
    @property
    def event_loop(self):
        if self._transport:
            return self._transport.event_loop
    
    @property
    def sock(self):
        if self._transport:
            return self._transport.sock
        
    @property
    def closing(self):
        return self._transport.closing if self._transport else True
    
    @property
    def closed(self):
        return self._transport.closed if self._transport else True
    
    def fileno(self):
        if self._transport:
            return self._transport.fileno()
    
    def is_stale(self):
        return self._transport.is_stale() if self._transport else True
    
    def close(self, async=True, exc=None):
        if self._transport:
            self._transport.close(async=async, exc=exc)
        
    def abort(self, exc=None):
        self.close(async=False, exc=exc)
        
        
class Connector(Deferred, TransportProxy):
    '''A :class:`Connector` is in place of the :class:`Connection` during
a connection operation. Once the connection with end-point is established, the
connector fires its callbacks.'''
    processed = 0
    def __init__(self, transport, timeout=None):
        self._transport = transport
        self._consumer = None
        super(Connector, self).__init__(timeout=timeout or 10,
                                        event_loop=transport.event_loop)
        self.add_callback(self._connection_made, self._connection_failure)
    
    def __repr__(self):
        return 'Connector for %s' % self._transport
        
    def _connection_made(self, result):
        self._transport._protocol.connection_made(self._transport)
        return self._transport._protocol
        
    def _connection_failure(self, failure):
        self._transport._protocol.connection_lost(failure)
        return failure
        
    def set_consumer(self, consumer):
        '''Set the :class:`ProtocolConsumer` which will consume the protocol
once the connection is made.'''
        consumer._connection = self
        # add callback and errorback to self
        self.add_callback(lambda c: c.set_consumer(consumer),
                          lambda f: consumer.connection_lost(f))
        
    def finished(self, consumer, exc=None):
        # Called by the protocol consumer when the connector is still in place
        # of the connection
        consumer.fire_event('finish', exc)
        # important, return the original result
        return exc
        


#!/usr/bin/env python
'''This a web chat application which illustrates how to run a django site with
pulsar and how to include pulsar asynchronous request middlewares into django.
Requires django 1.4 or above. To run::

    python manage.py pulse

If running for the first time, issue the::

    python manage.py syncdb

command and create the super user.
'''
import os, sys
try:
    import pulsar
except ImportError: #pragma nocover
    import sys
    sys.path.append('../../')
    
os.environ["DJANGO_SETTINGS_MODULE"] = "chat.settings"
from django.core.management import execute_from_command_line
    
if __name__ == "__main__":
    execute_from_command_line()


# This is for python 2.x
from inspect import istraceback
def raise_error_trace(err, traceback):
    if istraceback(traceback):
        raise err.__class__, err, traceback
    else:
        raise err.__class__, err, None


from time import time
import os
import sys
import signal
from multiprocessing import current_process

import pulsar
from pulsar.utils.tools import Pidfile
from pulsar.utils.security import gen_unique_id
from pulsar.utils.pep import itervalues, iteritems
from pulsar.utils.log import process_global
from pulsar import HaltServer, system

from .actor import Actor, ACTOR_STATES
from .monitor import PoolMixin, _spawn_actor
from .defer import multi_async, log_failure
from .access import get_actor, set_actor
from .mailbox import MailboxConsumer
from . import proxy


__all__ = ['arbiter', 'spawn', 'Arbiter']


def arbiter(commands_set=None, **params):
    '''Obtain the :class:`Arbiter` instance if we are on the arbiter
context domain, otherwise it returns ``None``.'''
    arbiter = get_actor()
    if arbiter is None:
        # Create the arbiter
        return set_actor(_spawn_actor(Arbiter, None, **params))
    elif isinstance(arbiter, Actor) and arbiter.is_arbiter():
        return arbiter

#TODO: why cfg is set to None?
def spawn(cfg=None, **kwargs):
    '''Spawn a new :class:`Actor` and return an :class:`ActorProxyDeferred`.
This method can be used from any :class:`Actor`.
If not in the :class:`Arbiter` domain,
the method send a request to the :class:`Arbiter` to spawn a new actor. Once
the arbiter creates the actor it returns the proxy to the original caller.

**Parameter kwargs**

These optional parameters are:
    * *actor_class* a custom :class:`Actor` subclass.
    * *aid* the actor id
    * *commands_set* the set of :ref:`remote commands <api-remote_commands>`
      the :class:`Actor` can respond to.
    
:rtype: an :class:`ActorProxyDeferred`.

A typical usage::

    >>> a = spawn()
    >>> a.aid
    'ba42b02b'
    >>> a.called
    True
    >>> p = a.result
    >>> p.address
    ('127.0.0.1', 46691)
    '''
    aid = gen_unique_id()[:8]
    kwargs['aid'] = aid
    actor = get_actor()
    # The actor is not the Arbiter domain.
    # We send a message to the Arbiter to spawn a new Actor
    if not isinstance(actor, Arbiter):
        # send the request to the arbiter
        msg = actor.send('arbiter', 'spawn', **kwargs)
        return proxy.ActorProxyDeferred(aid, msg)
    else:
        return actor.spawn(**kwargs)


def stop_arbiter(self):
    p = self.pidfile
    if p is not None:
        self.logger.debug('Removing %s' % p.fname)
        p.unlink()
    if self.managed_actors:
        self.state = ACTOR_STATES.TERMINATE
    self.logger.info("Bye.")
    if self.exit_code:
        sys.exit(self.exit_code)
    
def start_arbiter(self):
    if current_process().daemon:
        raise pulsar.PulsarException(
                'Cannot create the arbiter in a daemon process')
    os.environ["SERVER_SOFTWARE"] = pulsar.SERVER_SOFTWARE
    pidfile = self.cfg.pidfile
    if pidfile is not None:
        try:
            p = Pidfile(pidfile)
            p.create(self.pid)
        except RuntimeError as e:
            raise HaltServer(str(e))
        self.pidfile = p
    
    
class Arbiter(PoolMixin):
    '''The Arbiter is the most important a :class:`Actor`
and :class:`PoolMixin` in pulsar concurrent framework. It is used as singleton
in the main process and it manages one or more :class:`Monitor`.
It runs the main :class:`EventLoop` of your concurrent application.
It is the equivalent of the gunicorn_ arbiter, the twisted_ reactor
and the tornado_ eventloop.

Users access the arbiter (in the arbiter process domain) by the high level api::

    import pulsar

    arbiter = pulsar.arbiter()

.. _gunicorn: http://gunicorn.org/
.. _twisted: http://twistedmatrix.com/trac/
.. _tornado: http://www.tornadoweb.org/
'''
    pidfile = None
    
    def __init__(self, impl):
        super(Arbiter, self).__init__(impl)
        self.monitors = {}
        self.registered = {'arbiter': self}
        self.bind_event('start', start_arbiter)
        self.bind_event('stop', stop_arbiter)

    ############################################################################
    # ARBITER HIGH LEVEL API
    ############################################################################
    def is_arbiter(self):
        return True

    def add_monitor(self, monitor_class, monitor_name, **params):
        '''Add a new :class:`Monitor` to the :class:`Arbiter`.

:parameter monitor_class: a :class:`pulsar.Monitor` class.
:parameter monitor_name: a unique name for the monitor.
:parameter kwargs: dictionary of key-valued parameters for the monitor.
:rtype: an instance of a :class:`pulsar.Monitor`.'''
        if monitor_name in self.registered: 
            raise KeyError('Monitor "{0}" already available'\
                           .format(monitor_name))
        params['name'] = monitor_name
        m = self.spawn(monitor_class, **params)
        self.registered[m.name] = m
        self.monitors[m.aid] = m
        return m

    def is_process(self):
        return True

    def close_monitors(self):
        '''Close all :class:`Monitor` at once.'''
        return multi_async([m.stop() for m in list(itervalues(self.monitors))],
                           log_failure=True)

    def get_actor(self, aid):
        '''Given an actor unique id return the actor proxy.'''
        a = super(Arbiter, self).get_actor(aid)
        if a is None:
            if aid in self.monitors:    # Check in monitors aid
                return self.monitors[aid]
            elif aid in self.managed_actors:
                return self.managed_actors[aid]
            elif aid in self.registered:
                return self.registered[aid]
            else:    # Finally check in workers in monitors
                for m in itervalues(self.monitors):
                    if aid in m.managed_actors:
                        return m.managed_actors[aid]
        else:
            return a
    
    def info(self):
        data = super(Arbiter, self).info()
        monitors = {}
        for m in itervalues(self.monitors):
            info = m.info()
            actor = info['actor']
            monitors[actor['name']] = info
        server = data.pop('actor')
        server.update({'version': pulsar.__version__,
                       'name': pulsar.SERVER_NAME,
                       'number_of_monitors': len(self.monitors),
                       'number_of_actors': len(self.managed_actors)})
        server.pop('is_process', None)
        server.pop('ppid', None)
        server.pop('actor_id', None)
        server.pop('age', None)
        data['server'] = server
        data['workers'] = [a.info for a in itervalues(self.managed_actors)]
        data['monitors'] = monitors
        return data
    
    def identity(self):
        return self.name
    
    ############################################################################
    # INTERNALS
    ############################################################################
    def _remove_actor(self, actor, log=True):
        super(Arbiter, self)._remove_actor(actor, log)
        self.registered.pop(actor.name, None)
        self.monitors.pop(actor.aid, None)
        
    def periodic_task(self):
        # Arbiter periodic task
        if self.can_continue() and self.running():
            # managed actors job
            self.manage_actors()
            for m in list(itervalues(self.monitors)):
                if m.started():
                    if not m.running():
                        self._remove_actor(m)
                else:
                    m.start()
            try:
                self.cfg.arbiter_task(self)
            except Exception:
                pass
        self.event_loop.call_soon(self.periodic_task)

    def _stop(self):
        '''Stop the pools the message queue and remaining actors.'''
        self.event_loop.call_soon_threadsafe(self._exit)
        self._run(False)
        
    def _exit(self, res=None):
        if res:
            self.state = ACTOR_STATES.CLOSE
            self.mailbox.close()
        else:
            active = multi_async((self.close_monitors(), self.close_actors()),
                                 log_failure=True)
            active.add_both(self._exit)
                
    def start(self):
        if self.state == ACTOR_STATES.INITIAL:
            if self.cfg.daemon: #pragma    nocover
                system.daemonize()
            return Actor.start(self)
        
    def _mailbox(self, event_loop):
        address = ('127.0.0.1', 0)
        mailbox = event_loop.create_server(address=address,
                                           name='Mailbox for %s' % self,
                                           consumer_factory=MailboxConsumer,
                                           timeout=0,
                                           close_event_loop=True)
        event_loop.call_soon_threadsafe(self.hand_shake)
        return mailbox


import socket
from functools import partial, reduce
from threading import Lock

from pulsar import TooManyConnections
from pulsar.utils.pep import get_event_loop, new_event_loop, itervalues, range
from pulsar.utils.sockets import get_transport_type, create_socket
from pulsar.async.defer import is_failure, multi_async

from .protocols import ProtocolConsumer, EventHandler, Producer
from .transport import create_transport, LOGGER

__all__ = ['ConnectionPool', 'Client', 'Request', 'SingleClient']

    
class Request(object):
    '''A :class:`Client` request class is an hashable object used to select
the appropiate :class:`ConnectionPool` for the client request.'''
    def __init__(self, address, timeout=0):
        self.address = address
        self.timeout = timeout
        
    @property
    def key(self):
        return (self.address, self.timeout)
    
    def encode(self):
        raise NotImplementedError
    
    
class ConnectionPool(Producer):
    '''A :class:`Producer` of of active connections for client
protocols. It maintains a live set of connections.

.. attribute:: address

    Address to connect to
    '''    
    def __init__(self, request, **params):
        params['timeout'] = request.timeout
        self.lock = Lock()
        super(ConnectionPool, self).__init__(**params)
        self._address = request.address
        self._available_connections = set()
    
    def __repr__(self):
        return repr(self.address)
    
    def __str__(self):
        return self.__repr__()
        
    @property
    def address(self):
        return self._address
    
    @property
    def available_connections(self):
        return len(self._available_connections)
        
    def release_connection(self, connection, response=None):
        '''Releases the *connection* back to the pool. This function remove
the *connection* from the set of concurrent connections and add it to the set
of available connections.

:parameter connection: The connection to release
:parameter response: Optional :class:`ProtocolConsumer` which consumed the
    connection.
'''
        with self.lock:
            #LOGGER.debug('release connection: available %s, concurrent %s',
            #        self.available_connections, self.concurrent_connections)
            self._concurrent_connections.discard(connection)
            if connection.producer.can_reuse_connection(connection, response):
                self._available_connections.add(connection)
        
    def get_or_create_connection(self, client):
        "Get or create a new connection for *client*"
        stale_connections = []
        with self.lock:
            try:
                closed = True
                while closed:
                    connection = self._available_connections.pop()
                    closed = connection.is_stale()
                    if closed and not connection.closed:
                        stale_connections.append(connection)
            except KeyError:
                connection = None
            else:
                # we have a connection, lets added it to the concurrent set
                self._concurrent_connections.add(connection)
        for sc in stale_connections:
            sc.transport.close()
        if connection is None:
            # build the new connection
            connection = self.new_connection(self.address,
                                             client.consumer_factory,
                                             producer=client)
            # Bind the post request event to the release connection function
            connection.bind_event('post_request', self._release_response)
            # Bind the connection_lost to connection to handle dangling connections
            connection.bind_event('connection_lost',
                                  partial(self._try_reconnect, connection))
            #IMPORTANT: create client transport an connect to endpoint
            transport = create_transport(connection, address=connection.address,
                                         event_loop=client.get_event_loop())
            return transport.connect(connection.address)
        else:
            return connection
        
    ############################################################################
    ##    INTERNALS
    def _try_reconnect(self, connection, exc):
        # handle Read Exception on the transport
        if is_failure(exc) and exc.is_instance((socket.timeout, socket.error)):
            # Have we been here before?
            consumer = connection.current_consumer
            if consumer is None:
                # No consumer, The address was probably wrong. Connection Refused
                return
            client = connection.producer
            received = getattr(consumer, '_received_count', -1)
            # The connection has processed request before and the consumer
            # has never received data. If the client allows it, try to
            # reconnect, it was probably a stale connection.
            retries = consumer.can_reconnect(client.max_reconnect, exc)
            if retries:
                connection._current_consumer = None
                lag = retries - 1
                if lag:
                    lag = client.reconnect_time_lag(lag)
                    LOGGER.debug('Try to reconnect in %s seconds', lag)
                    loop = get_event_loop()
                    loop.call_later(lag, self._reconnect, client, consumer)
                else:
                    LOGGER.debug('Try to reconnect')
                    self._reconnect(client, consumer)
                    
    def _reconnect(self, client, consumer):
        # get a new connection
        conn = self.get_or_create_connection(client)
        # Start the response without firing the events
        conn.set_consumer(consumer)
        consumer.new_request(consumer.current_request)
                
    def _remove_connection(self, connection, exc=None):
        with self.lock:
            super(ConnectionPool, self)._remove_connection(connection, exc)
            self._available_connections.discard(connection)
    
    def _release_response(self, response):
        #proxy to release_connection
        if getattr(response, 'release_connection', True):
            self.release_connection(response.connection, response)


class Client(EventHandler):
    '''A client for a remote server which handles one or more
:class:`ConnectionPool` of asynchronous connections.
It has the ``finish`` :ref:`one time event <one-time-event>` fired when calling
the :meth:`close` method, four :ref:`many time events <many-times-event>`
which can be used to add additional information to the request to send
to the remote server and to postprocess responses. These events are:
``connection_made``, ``pre_request``, ``post_request``, ``connection_lost``.

:param max_connections: Optional maximum number of connections.
:param timeout: Optional timeout in seconds for closing idle connections.
:param force_sync: set the :attr:`force_sync` attribute.
:param event_loop: Optional :class:`EventLoop` which set the :attr:`event_loop`.

.. attribute:: event_loop

    The :class:`EventLoop` for this :class:`Client`. Can be ``None``.
    
.. attribute:: force_sync

    Force a synchronous client, that is a client which has it
    own :class:`EventLoop` and blocks until a response is available.
    
    Default: `False`
'''
    max_reconnect = 1
    '''Can reconnect on socket error.'''
    connection_pools = None
    '''Dictionar of :class:`ConnectionPool`. If initialized at class level it
will remain as a class attribute, otherwise it will be an instance attribute.'''
    connection_pool = ConnectionPool
    '''Factory of :class:`ConnectionPool`.'''
    consumer_factory = None
    '''A factory of :class:`ProtocolConsumer` for sending and consuming data.'''
    connection_factory = None
    '''A factory of :class:`Connection`.'''
    client_version = ''
    '''An optional version for this client'''
    timeout = 0
    '''Optional timeout in seconds for idle connections.'''
    max_connections = 0
    '''Maximum number of concurrent connections.'''
    reconnecting_gap = 2
    '''Reconnecting gap in seconds.'''
    
    
    ONE_TIME_EVENTS = ('finish',)
    MANY_TIMES_EVENTS = ('connection_made', 'pre_request','post_request',
                         'connection_lost')
    
    def __init__(self, max_connections=None, timeout=None, client_version=None,
                 trust_env=True, consumer_factory=None, max_reconnect=None,
                 force_sync=False, event_loop=None, **params):
        super(Client, self).__init__()
        self.lock = Lock()
        self.trust_env = trust_env
        self.client_version = client_version or self.client_version
        self.timeout = timeout if timeout is not None else self.timeout
        if consumer_factory:
            self.consumer_factory = consumer_factory
        self.max_connections = max_connections or self.max_connections or 2**31
        if self.connection_pools is None:
            self.connection_pools = {}
        if max_reconnect:
            self.max_reconnect = max_reconnect
        self.force_sync = force_sync
        self.event_loop = event_loop
        self.setup(**params)
    
    def setup(self, **params):
        '''Setup the client. By default it does nothing.'''
        pass
    
    def __str__(self):
        return self.__repr__()
    
    def __repr__(self):
        return self.__class__.__name__
    
    @property
    def concurrent_connections(self):
        '''Total number of concurrent connections.'''
        return reduce(lambda x,y: x + y, (p.concurrent_connections for p in\
                                          itervalues(self.connection_pools)), 0)
    
    @property
    def available_connections(self):
        '''Total number of available connections.'''
        return reduce(lambda x,y: x + y, (p.available_connections for p in\
                                          itervalues(self.connection_pools)), 0)
        
    def get_event_loop(self):
        '''Return the :class:`EventLoop` used by this :class:`Client`.
The event loop can be set during initialisation. If :attr:`force_sync`
is ``True`` a specialised event loop is created.'''
        if self.event_loop:
            return self.event_loop
        elif self.force_sync:
            self.event_loop = new_event_loop(iothreadloop=False)
            return self.event_loop
        else:
            return get_event_loop()
    
    def hash(self, address, timeout, request):
        return hash((address, timeout))
    
    def request(self, *args, **params):
        '''Abstract method for creating a request to send to the server.
**Must be implemented by subclasses**. The method should return a
:class:`ProtocolConsumer` via invoking the :meth:`response` method. '''
        raise NotImplementedError
    
    def response(self, request):
        '''Once a *request* object has been constructed, the :meth:`request`
method can invoke this method to build the protocol consumer and
start the response.

:parameter request: A custom :class:`Request` for the :class:`Client`.
:rtype: An :class:`ProtocolConsumer` obtained form
    :attr:`consumer_factory`.
'''
        conn = self.get_connection(request)
        # build the protocol consumer
        consumer = self.consumer_factory(conn)
        # get the event loop
        event_loop = self.get_event_loop()
        # start the request in the event-loop thread
        event_loop.call_soon_threadsafe(consumer.new_request, request)
        #consumer.new_request(request)
        if self.force_sync: # synchronous response
            event_loop.run_until_complete(consumer.on_finished)
        return consumer
    
    def get_connection(self, request):
        '''Get a suitable :class:`Connection` for *request*.'''
        with self.lock:
            pool = self.connection_pools.get(request.key)
            if pool is None:
                pool = self.connection_pool(
                                request,
                                max_connections=self.max_connections,
                                connection_factory=self.connection_factory)
                self.connection_pools[request.key] = pool
        return pool.get_or_create_connection(self)
        
    def update_parameters(self, parameter_list, params):
        '''Update *param* with attributes of this :class:`Client` defined
in :attr:`request_parameters` tuple.'''
        for name in parameter_list:
            if name not in params:
                params[name] = getattr(self, name)
        return params
        
    def close_connections(self, async=True):
        '''Close all connections in each :attr:`connection_pools`.'''
        for p in self.connection_pools.values():
            p.close_connections(async=async)
            
    def close(self, async=True):
        '''Close all connections and fire the ``finish``
:ref:`one time event <one-time-event>`'''
        self.close_connections(async)
        self.fire_event('finish')
        
    def abort(self):
        self.close(async=False)

    def can_reuse_connection(self, connection, response):
        '''Invoked by the :meth:`ConnectionPool.release_connection`, it checks
whether the *connection* can be reused in the future or it must be disposed.

:param connection: the :class:`Connection` to check.
:param response: the :class:`ProtocolConsumer` which last consumed the incoming
    data from the connection (it can be ``None``).
:return: ``True`` or ``False``.
'''
        return True
    
    def reconnect_time_lag(self, lag):
        lag = self.reconnect_time_lag*(math.log(lag) + 1)
        return round(lag, 1)
    
    def remove_pool(self, pool):
        key = None
        for key, p in self.connection_pools.items():
            if pool is p:
                break
        if key:
            self.connection_pools.pop(key)
            
    def upgrade(self, connection, protocol_factory, result=None):
        '''Upgrade an existing connection with a new protocol factory.
Return the upgraded connection only if the :attr:`Connection.current_consumer`
is available.'''
        protocol = connection.current_consumer
        if protocol:
            protocol.release_connection = False
            protocol.finished(result)
            connection.upgrade(protocol_factory)
            return connection
    
    def timeit(self, times, *args, **kwargs):
        '''Send *times* requests asynchronously and evaluate the time
taken to obtain all responses.'''
        results = []
        for _ in range(times):
            r = self.request(*args, **kwargs)
            if hasattr(r, 'on_finished'):
                r = r.on_finished
            results.append(r)
        return multi_async(results)
             
        
class SingleClient(Client):
    '''A :class:`Client` which handle one connection only.'''
    def __init__(self, address, **kwargs):
        super(SingleClient, self).__init__(**kwargs)
        self.address = address
        self._consumer = None
    
    def response(self, request):
        if self._consumer is None:
            self._consumer = super(SingleClient, self).response(request)
        else:
            self._consumer.new_request(request)
        return self._consumer